#!/bin/sh

#      debian.sh
#      
#      Copyright 2009-2010 Alex <alex@linuxonly.ru>
#      
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 3 of the License, or
#      (at your option) any later version.
#      
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#      
#      You should have received a copy of the GNU General Public License
#      along with this program. If not, see <http://www.gnu.org/licenses/>.


PACKAGENAME="gsf"
VERSION="0.2.0"
RELEASE="1"
BUILDPREFIX="/tmp"

if test `uname -m` = "x86_64"; then ARCH="amd64"
else ARCH=`uname -m|sed 's/^i[3-6]86/i386/'`
fi

DEBNAME=${PACKAGENAME}-${VERSION}-${RELEASE}_${ARCH}

echo Building $DEBNAME.deb at ${BUILDPREFIX}/${DEBNAME}

rm -rf ${BUILDPREFIX}/${DEBNAME}
mkdir -p ${BUILDPREFIX}/${DEBNAME}

mkdir -p ${BUILDPREFIX}/${DEBNAME}/DEBIAN
cat > ${BUILDPREFIX}/${DEBNAME}/DEBIAN/control << END
Package: $PACKAGENAME
Version: ${VERSION}-${RELEASE}
Section: utils
Priority: optional
Architecture: $ARCH
Installed-Size: 536
Depends: binutils, gcc, libc6, libgtk2.0-0, libgdbm3, zlib1g, libasound2, libpulse0
Provides: gsf
Recommends: gsf-sounds-en
Suggests: gsf-sounds-en
Maintainer: Alex <alex@linuxonly.ru>
Homepage: http://linuxonly.ru
Description: GSF software satfinder (http://linuxonly.ru)
 GSF provides simple way for satellite antenna tuning.
END

cat > ${BUILDPREFIX}/${DEBNAME}/DEBIAN/postinst << END
#!/bin/sh
set -e
# Automatically added by dh_installmenu
if [ "$1" = "configure" ] && [ -x "`which update-menus 2>/dev/null`" ]; then
	update-menus
fi
#SUID bit for privileged process
chmod 4755 /usr/bin/gsfprivileged
# End automatically added section
END

cat > ${BUILDPREFIX}/${DEBNAME}/DEBIAN/postrm << END
#!/bin/sh
set -e
# Automatically added by dh_installmenu
if [ -x "`which update-menus 2>/dev/null`" ]; then update-menus ; fi
# End automatically added section
END

mkdir -p ${BUILDPREFIX}/${DEBNAME}/usr/bin
mkdir -p ${BUILDPREFIX}/${DEBNAME}/usr/lib
mkdir -p ${BUILDPREFIX}/${DEBNAME}/usr/share
mkdir -p ${BUILDPREFIX}/${DEBNAME}/usr/share/pixmaps
mkdir -p ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf

cat > ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/copyright << END
Upstream Author(s):

    Alex

Copyright:

    <Copyright (C) 2009 - 2010  Alex>

License:

    GNU General Public License 3

The Debian packaging is:

    Copyright (C) 2010 Alex <alex@linuxonly.ru>

and is licensed under the GPL version 3,
see '/usr/share/common-licenses/GPL-3'.
END

cat > ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/changelog << END
gsf (0.2.0-1) stable; urgency=low

  * Initial release

 -- Alex <alex@linuxonly.ru>  Wed, 22 Dec 2010 23:06:50 +0400

END

gzip -f --best ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/changelog
mv ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/changelog.gz  ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/changelog.Debian.gz

touch ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/AUTHORS
touch ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/README
touch ${BUILDPREFIX}/${DEBNAME}/usr/share/doc/gsf/NEWS.gz

mkdir -p ${BUILDPREFIX}/${DEBNAME}/usr/share/menu
cat > ${BUILDPREFIX}/${DEBNAME}/usr/share/menu/gsf << END
?package(gsf):needs="X11" \\
  section="Apps/Utils" \\
  title="GSF" \\
  longtitle="GSF - Software satfinder for GNU/Linux" \\
  command="/usr/bin/gsf"
END

mkdir -p ${BUILDPREFIX}/${DEBNAME}/usr/share/applications
cat > ${BUILDPREFIX}/${DEBNAME}/usr/share/applications/gsf.desktop << END
[Desktop Entry]
Name=GSF
GenericName=Software satfinder
Comment=Tune satellite antenna and manipulate DVB transport streams
Categories=Utility;
Exec=gsf
Icon=/usr/share/pixmaps/gsf.png
StartupNotify=true
Terminal=false
Type=Application
X-Ubuntu-Gettext-Domain=gsf
END

echo '[+] Building program';
(cd ..; sh configure --prefix=/usr)
(cd ..; export INSTALLPREFIX=${BUILDPREFIX}/${DEBNAME}/; make && make install)

(cd ${BUILDPREFIX}/${DEBNAME}; find usr -type f -exec md5sum {} >> ./DEBIAN/md5sums \;)

chmod -R 0755 ${BUILDPREFIX}/${DEBNAME}/DEBIAN

rm -f $DEBNAME.deb

echo '[+] Building debian package';
fakeroot dpkg-deb --build ${BUILDPREFIX}/${DEBNAME}

echo '[+] Cleaning up source tree';
(cd ..; make clean)
rm -rf ${BUILDPREFIX}/${DEBNAME}

echo '[+] Finished';
exit;

