/*
 *      calcwindow.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include "settings.h"
#include "calculations.h"
#include "resources.h"

struct scalcwindow {
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	
	GtkWidget *sitelong;
	GtkWidget *sitelat;
	GtkWidget *satlong;
	GtkWidget *intseason;
	GtkWidget *intyear;
	GtkWidget *intband;
	GtkWidget *intantsize;
	
	GtkWidget *satazimuth;
	GtkWidget *sataltitude;
	GtkWidget *satpolangle;
	GtkWidget *satdistance;
	
	GtkWidget *sunazimuth;
	GtkWidget *sunaltitude;
	GtkWidget *sunsat;
	GtkWidget *moonazimuth;
	GtkWidget *moonaltitude;
	GtkWidget *moonsat;
	
	GtkWidget *intlist;
	GtkListStore *store;
	GtkWidget *calcbutton;
	GtkWidget *clearbutton;
	
	guint timeout;
};
struct scalcwindow calcwindow;

struct _values {
	gdouble sitelat;
	gdouble sitelong;
	gdouble satlong;
	gint season;
	gint year;
	gint band;
	gdouble antsize;
		
	gdouble sunazimuth;
	gdouble sunaltitude;
	gdouble sunsat;
	gdouble moonazimuth;
	gdouble moonaltitude;
	gdouble moonsat;
	
} values;

enum {
	INTERFERENCE_PERIOD = 0,
	INTERFERENCE_LENGTH
};

void calcwindow_sitepos_changed(GtkSpinButton *spinbutton, gpointer data);
gboolean calcwindow_update_tracker(gpointer data);
void calcwindow_interference_clear(GtkButton *button, gpointer data);
void calcwindow_interference_foreach(intperiod *data, gpointer user_data);
gint calcwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
void calcwindow_interference_count(GtkButton *button, gpointer data);

void calcwindow_sitepos_changed(GtkSpinButton *spinbutton, gpointer data)
{
	float az, el, pol, sh;
	gchar strbuf[256];
	
	values.sitelong = gtk_spin_button_get_value(GTK_SPIN_BUTTON(calcwindow.sitelong));
	values.sitelat = gtk_spin_button_get_value(GTK_SPIN_BUTTON(calcwindow.sitelat));
	values.satlong = gtk_spin_button_get_value(GTK_SPIN_BUTTON(calcwindow.satlong));
	values.season = gtk_combo_box_get_active(GTK_COMBO_BOX(calcwindow.intseason));
	values.year = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(calcwindow.intyear));
	values.band = gtk_combo_box_get_active(GTK_COMBO_BOX(calcwindow.intband));
	values.antsize = gtk_spin_button_get_value(GTK_SPIN_BUTTON(calcwindow.intantsize));
	
	
	az = azimuth(values.sitelong, values.sitelat, values.satlong, 200.0);
	el = elevation(values.sitelong, values.sitelat, values.satlong, 200.0);
	pol = polangle(values.sitelong, values.sitelat, values.satlong); 
	sh = satheight(values.sitelong, values.sitelat, values.satlong);
	
	if (calcwindow.visible) {
		g_sprintf(strbuf, "%'.3f°", az);
		gtk_label_set_label(GTK_LABEL(calcwindow.satazimuth), strbuf);
		g_sprintf(strbuf, "%'.3f°", el);
		gtk_label_set_label(GTK_LABEL(calcwindow.sataltitude), strbuf);
		g_sprintf(strbuf, "%'.3f°", pol);
		gtk_label_set_label(GTK_LABEL(calcwindow.satpolangle), strbuf);
		g_sprintf(strbuf, "%'.3f km", sh);
		gtk_label_set_label(GTK_LABEL(calcwindow.satdistance), strbuf);
	}
	
}

gboolean calcwindow_update_tracker(gpointer data)
{
	time_t timestamp;
	gchar strbuf[256];
	struct tm *timespec;
	
	if (calcwindow.visible) {
		timestamp = time(NULL);
		timespec = localtime(&timestamp);
		timestamp = mktime(timespec);
		values.sunsat = 1000.0;
		values.moonsat = 1000.0;
		tracker(values.sitelong, values.sitelat, timestamp, &values.sunazimuth, &values.sunaltitude, &values.sunsat, &values.moonazimuth, &values.moonaltitude, &values.moonsat);
		
		if (calcwindow.visible) {
			g_sprintf(strbuf, "%'.3f°", values.sunazimuth);
			gtk_label_set_label(GTK_LABEL(calcwindow.sunazimuth), strbuf);
			g_sprintf(strbuf, "%'.3f°", values.sunaltitude);
			gtk_label_set_label(GTK_LABEL(calcwindow.sunaltitude), strbuf);
			if (values.sunsat == 1000.0) g_sprintf(strbuf, "N/A"); else g_sprintf(strbuf, "%'.3f°", values.sunsat);
			gtk_label_set_label(GTK_LABEL(calcwindow.sunsat), strbuf);
			
			g_sprintf(strbuf, "%'.3f°", values.moonazimuth);
			gtk_label_set_label(GTK_LABEL(calcwindow.moonazimuth), strbuf);
			g_sprintf(strbuf, "%'.3f°", values.moonaltitude);
			gtk_label_set_label(GTK_LABEL(calcwindow.moonaltitude), strbuf);
			if (values.moonsat == 1000.0) g_sprintf(strbuf, "N/A"); else g_sprintf(strbuf, "%'.3f°", values.moonsat);
			gtk_label_set_label(GTK_LABEL(calcwindow.moonsat), strbuf);
		}
		
		return TRUE; 
	}
	
	return FALSE;
}

gint calcwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
    calcwindow.visible = FALSE;
	
	settings_append_float("calc_site_lat", values.sitelat);
	settings_append_float("calc_site_long", values.sitelong);
	settings_append_float("calc_sat_long", values.satlong);
	settings_append_int("calc_int_season", values.season);
	settings_append_int("calc_int_year", values.year);
	settings_append_int("calc_int_band", values.band);
	settings_append_float("calc_int_antsize", values.antsize);
	settings_flush();
	
	if (calcwindow.timeout > 0) {
		g_source_remove(calcwindow.timeout);
		calcwindow.timeout = 0;
	}
	
	gtk_widget_destroy(calcwindow.win);
    return FALSE;
}

void calcwindow_interference_clear(GtkButton *button, gpointer data)
{
	gtk_list_store_clear(calcwindow.store);
}

void calcwindow_interference_foreach(intperiod *data, gpointer user_data)
{
	GtkTreeIter iter;
	gchar strbuf[256], strbuf1[16], strbuf2[16], strbuf3[16];
	
	
	if (strftime(strbuf1, sizeof(strbuf1), "%D", localtime(&data->start)) == 0) {
		g_sprintf(strbuf1, "unknown");
	}
	
	if (strftime(strbuf2, sizeof(strbuf2), "%T %Z", localtime(&data->start)) == 0) {
		g_sprintf(strbuf2, "unknown");
	}
	
	if (strftime(strbuf3, sizeof(strbuf3), "%T %Z", localtime(&data->end)) == 0) {
		g_sprintf(strbuf3, "unknown");
	}
	
	g_snprintf(strbuf, sizeof(strbuf), "%s - %s - %s", strbuf1, strbuf2, strbuf3);
	
	gtk_list_store_append(calcwindow.store, &iter);
	gtk_list_store_set(calcwindow.store, &iter, INTERFERENCE_PERIOD, strbuf, -1);
	
}

void calcwindow_interference_count(GtkButton *button, gpointer data)
{
	GSList *reslist = NULL;
	
	reslist = interferention_main(values.sitelong, values.sitelat, values.sitelong, values.year, values.season, values.band, values.antsize);
	
	if (reslist != NULL) {
		gtk_list_store_clear(calcwindow.store);
		//printf("%i\n", g_slist_length(reslist));
		g_slist_foreach(reslist, (GFunc)calcwindow_interference_foreach, NULL);
		g_slist_free(reslist);
	}
}

gboolean calcwindow_create(void)
{
    GtkWidget *vbox, *hbox;
	GtkWidget *align;
	GtkWidget *gentable, *aptable, *trtable, *inttable;
	GtkWidget *sitecoord_l, *satcoord_l, *satazimuth_l, *sataltitude_l,
	          *satpolangle_l, *satdistance_l, *sunazimuth_l, *sunaltitude_l,
			  *sunsat_l, *moonazimuth_l, *moonaltitude_l, *moonsat_l,
			  *season_l, *antenna_l;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *evbox;
	GtkWidget *chbox;
	GtkWidget *icon;
	GtkWidget *scrollwin;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *notebook;
	GtkWidget *cdata_cp;
	GtkWidget *ap_cp;
	GtkWidget *tr_cp;
	GtkWidget *int_cp;
	time_t t;
	struct tm *tmp;
	
	if (!calcwindow.visible) {
	  calcwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  gtk_window_set_modal(GTK_WINDOW(calcwindow.win), TRUE);
	  gtk_window_set_resizable(GTK_WINDOW(calcwindow.win), FALSE);
	  calcwindow.title = _("Calculations");
	  gtk_window_set_position(GTK_WINDOW(calcwindow.win), GTK_WIN_POS_CENTER);
	  gtk_window_set_title(GTK_WINDOW(calcwindow.win), calcwindow.title);
	  gtk_widget_set_size_request(calcwindow.win, 400, 300);
	  gtk_window_set_default_size(GTK_WINDOW(calcwindow.win), 400, 300);
	  
	  vbox = gtk_vbox_new(FALSE, 0);
	  
	  fds = pango_font_description_from_string("14");
	  pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
	  calcwindow.caption = gtk_label_new(_("Calculations"));
	  gtk_widget_modify_font(calcwindow.caption, fds);
	  pango_font_description_free(fds);
	  evbox = gtk_event_box_new();
	  gdk_color_parse("white", &color);
	  gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
	  gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox),TRUE);
	  chbox = gtk_hbox_new(FALSE,0);
	  gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
	  icon = gtk_image_new_from_file(RESOURCE_CALCWINDOW_HEADER);
	  //icon = gtk_image_new_from_stock(GTK_STOCK_CONVERT, GTK_ICON_SIZE_DIALOG);
	  gtk_box_pack_start(GTK_BOX(chbox),icon,FALSE,FALSE,15);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align),calcwindow.caption);
	  gtk_box_pack_start(GTK_BOX(chbox),align,FALSE,FALSE,0);
	  gtk_container_add(GTK_CONTAINER(evbox),chbox);
	  
	  gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
	  
	  gentable = gtk_table_new(6, 3, FALSE);
	  //gtk_table_set_row_spacings(GTK_TABLE(gentable), 5);
	  
	  sitecoord_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(sitecoord_l),_("<b>Site latitude/longitude:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), sitecoord_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 0, 1);
	  
	  calcwindow.sitelat = gtk_spin_button_new_with_range(-90.0, 90.0, 1);
	  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(calcwindow.sitelat), 2);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(calcwindow.sitelat), settings_read_float("calc_site_lat", 55.75));
	  g_signal_connect(G_OBJECT(calcwindow.sitelat), "value-changed", G_CALLBACK(calcwindow_sitepos_changed), NULL);	  
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.sitelat);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 0, 1);
	  
	  calcwindow.sitelong = gtk_spin_button_new_with_range(-180.0, 180.0, 1);
	  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(calcwindow.sitelong), 2);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(calcwindow.sitelong), settings_read_float("calc_site_long", 37.70));
	  g_signal_connect(G_OBJECT(calcwindow.sitelong), "value-changed", G_CALLBACK(calcwindow_sitepos_changed), NULL);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.sitelong);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 2, 3, 0, 1);
	  
	  satcoord_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(satcoord_l),_("<b>Satellite longitude:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), satcoord_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 1, 2);
	  
	  calcwindow.satlong = gtk_spin_button_new_with_range(-180.0, 180.0, 1);
	  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(calcwindow.satlong), 2);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(calcwindow.satlong), settings_read_float("calc_sat_long", 60.00));
	  g_signal_connect(G_OBJECT(calcwindow.satlong), "value-changed", G_CALLBACK(calcwindow_sitepos_changed), NULL);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.satlong);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 3, 1, 2);
	  
	  t = time(NULL);
      tmp = localtime(&t);
	  
	  season_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(season_l),_("<b>Season:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), season_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 2, 3);
	  
	  calcwindow.intseason = gtk_combo_box_new_text();
	  gtk_combo_box_append_text(GTK_COMBO_BOX(calcwindow.intseason),_("Spring"));
	  gtk_combo_box_append_text(GTK_COMBO_BOX(calcwindow.intseason),_("Fall"));
	  gtk_combo_box_set_active(GTK_COMBO_BOX(calcwindow.intseason), settings_read_int("calc_int_season", (tmp == NULL) ? 0 : (tmp->tm_mon > 5) ? 1 : 0));
	  g_signal_connect(G_OBJECT(calcwindow.intseason), "changed", G_CALLBACK(calcwindow_sitepos_changed), NULL);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.intseason);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 2, 3);
	  
	  calcwindow.intyear = gtk_spin_button_new_with_range(1900, 2100, 1);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(calcwindow.intyear), settings_read_int("calc_int_year", (tmp == NULL) ? 2010 : tmp->tm_year+1900));
	  g_signal_connect(G_OBJECT(calcwindow.intyear), "value-changed", G_CALLBACK(calcwindow_sitepos_changed), NULL);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.intyear);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 2, 3, 2, 3);
	  
	  antenna_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(antenna_l),_("<b>Antenna:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), antenna_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 0, 1, 3, 4);
	  
	  calcwindow.intband = gtk_combo_box_new_text();
	  gtk_combo_box_append_text(GTK_COMBO_BOX(calcwindow.intband),_("C - Band"));
	  gtk_combo_box_append_text(GTK_COMBO_BOX(calcwindow.intband),_("Ku - Band"));
	  gtk_combo_box_set_active(GTK_COMBO_BOX(calcwindow.intband), settings_read_int("calc_int_band", 0));
	  g_signal_connect(G_OBJECT(calcwindow.intband), "changed", G_CALLBACK(calcwindow_sitepos_changed), NULL);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.intband);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 1, 2, 3, 4);
	  
	  calcwindow.intantsize = gtk_spin_button_new_with_range(0.6, 5.0, 1);
	  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(calcwindow.intantsize), 2);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(calcwindow.intantsize), settings_read_float("calc_int_antsize", 1.2));
	  g_signal_connect(G_OBJECT(calcwindow.intantsize), "value-changed", G_CALLBACK(calcwindow_sitepos_changed), NULL);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.intantsize);
	  gtk_table_attach_defaults(GTK_TABLE(gentable), align, 2, 3, 3, 4);
	  
	  aptable = gtk_table_new(4, 2, FALSE);
	  
	  satazimuth_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(satazimuth_l), _("<b>Satellite azimuth:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), satazimuth_l);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 0, 1, 0, 1);
	  
	  calcwindow.satazimuth = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.satazimuth), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.satazimuth);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 1, 2, 0, 1);
	  
	  sataltitude_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(sataltitude_l), _("<b>Satellite altitude:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), sataltitude_l);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 0, 1, 1, 2);
	  
	  calcwindow.sataltitude = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.sataltitude), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.sataltitude);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 1, 2, 1, 2);
	  
	  satpolangle_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(satpolangle_l), _("<b>Satellite pol. angle:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), satpolangle_l);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 0, 1, 2, 3);
	  
	  calcwindow.satpolangle = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.satpolangle), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.satpolangle);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 1, 2, 2, 3);
	  
	  satdistance_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(satdistance_l), _("<b>Satellite distance:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), satdistance_l);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 0, 1, 3, 4);
	  
	  calcwindow.satdistance = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.satdistance), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.satdistance);
	  gtk_table_attach_defaults(GTK_TABLE(aptable), align, 1, 2, 3, 4);
	  
	  
	  trtable = gtk_table_new(6, 2, FALSE);
	  
	  sunazimuth_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(sunazimuth_l), _("<b>Sun azimuth:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), sunazimuth_l);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 0, 1, 0, 1);
	  
	  calcwindow.sunazimuth = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.sunazimuth), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.sunazimuth);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 1, 2, 0, 1);
	  
	  sunaltitude_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(sunaltitude_l), _("<b>Sun altitude:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), sunaltitude_l);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 0, 1, 1, 2);
	  
	  calcwindow.sunaltitude = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.sunaltitude), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.sunaltitude);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 1, 2, 1, 2);
	  
	  sunsat_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(sunsat_l), _("<b>Sun - tuned satellite:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), sunsat_l);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 0, 1, 2, 3);
	  
	  calcwindow.sunsat = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.sunsat), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.sunsat);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 1, 2, 2, 3);
	  
	  
	  moonazimuth_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(moonazimuth_l), _("<b>Moon azimuth:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), moonazimuth_l);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 0, 1, 3, 4);
	  
	  calcwindow.moonazimuth = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.moonazimuth), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.moonazimuth);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 1, 2, 3, 4);
	  
	  moonaltitude_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(moonaltitude_l), _("<b>Moon altitude:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), moonaltitude_l);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 0, 1, 4, 5);
	  
	  calcwindow.moonaltitude = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.moonaltitude), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.moonaltitude);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 1, 2, 4, 5);
	  
	  moonsat_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(moonsat_l), _("<b>Moon - tuned satellite:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), moonsat_l);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 0, 1, 5, 6);
	  
	  calcwindow.moonsat = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(calcwindow.moonsat), _("--"));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), calcwindow.moonsat);
	  gtk_table_attach_defaults(GTK_TABLE(trtable), align, 1, 2, 5, 6);
	  
	  
	  inttable = gtk_table_new(2, 1, FALSE);
	  gtk_table_set_row_spacings(GTK_TABLE(inttable), 3);
	  
	  scrollwin = gtk_scrolled_window_new(NULL, NULL);
	  calcwindow.intlist = gtk_tree_view_new();
	  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin), GTK_SHADOW_ETCHED_IN);
	  renderer = gtk_cell_renderer_text_new();
	  column = gtk_tree_view_column_new_with_attributes("Period", renderer, "text", INTERFERENCE_PERIOD, NULL);
	  gtk_tree_view_append_column(GTK_TREE_VIEW(calcwindow.intlist), column);
	  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(calcwindow.intlist), FALSE);
	  gtk_container_add(GTK_CONTAINER(scrollwin), calcwindow.intlist);
	  gtk_table_attach_defaults(GTK_TABLE(inttable), scrollwin, 0, 2, 0, 1);
	  
	  calcwindow.store = gtk_list_store_new(INTERFERENCE_LENGTH, G_TYPE_STRING);
	  gtk_tree_view_set_model(GTK_TREE_VIEW(calcwindow.intlist), GTK_TREE_MODEL(calcwindow.store));
	  g_object_unref(calcwindow.store);
	  
	  calcwindow.calcbutton = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
	  gtk_table_attach(GTK_TABLE(inttable), calcwindow.calcbutton, 0, 1, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	  g_signal_connect(G_OBJECT(calcwindow.calcbutton), "clicked", G_CALLBACK(calcwindow_interference_count), NULL);	  
	  
	  calcwindow.clearbutton = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
	  gtk_table_attach(GTK_TABLE(inttable), calcwindow.clearbutton, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	  g_signal_connect(G_OBJECT(calcwindow.clearbutton), "clicked", G_CALLBACK(calcwindow_interference_clear), NULL);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(gentable), 10);
	  gtk_container_set_border_width(GTK_CONTAINER(aptable), 10);
	  gtk_container_set_border_width(GTK_CONTAINER(trtable), 10);
	  gtk_container_set_border_width(GTK_CONTAINER(inttable), 10);
	  
	  notebook = gtk_notebook_new();
	  cdata_cp = gtk_label_new(_("Data"));
	  ap_cp = gtk_label_new(_("Pointing"));
	  tr_cp = gtk_label_new(_("Tracker"));
	  int_cp = gtk_label_new(_("Interferention"));
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), gentable, cdata_cp);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), aptable, ap_cp);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), trtable, tr_cp);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), inttable, int_cp);
	  
	  gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
	  
	  calcwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	  	  	  
	  hbox = gtk_hbox_new(FALSE, 0);
	  gtk_box_pack_end(GTK_BOX(hbox), calcwindow.close, FALSE, FALSE, 5);
	  
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);

      g_signal_connect(G_OBJECT(calcwindow.win), "destroy",
                       G_CALLBACK(calcwindow_close), G_OBJECT(calcwindow.win));
	  
	  g_signal_connect(G_OBJECT(calcwindow.close), "clicked",
                       G_CALLBACK(calcwindow_close), G_OBJECT(calcwindow.win));
					   
	  gtk_container_add(GTK_CONTAINER(calcwindow.win),vbox);
	  gtk_widget_show_all(calcwindow.win);
	  
	  gtk_window_set_focus(GTK_WINDOW(calcwindow.win), calcwindow.close);
	  
	  calcwindow.timeout = g_timeout_add(1000, calcwindow_update_tracker, NULL);
	  
	  calcwindow.visible = TRUE;
	  
	  g_signal_emit_by_name(G_OBJECT(calcwindow.sitelat), "value-changed", NULL);
	  	  
  } else {
	  
	  gtk_window_present(GTK_WINDOW(calcwindow.win));
	  
  }
	  
      return TRUE; 
}
