/*
 *      devwindow.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
//#include "settings.h"
#include "mainwindow.h"
#include "dbstorage.h"
#include "dvbname.h"

struct sdevwindow {
	//device setup window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *apply;
	GtkWidget *restart;
	GtkWidget *device;
	GtkWidget *name, *vendor, *kdriver, *slot;
	GtkWidget *restart_ch;
	GtkWidget *drvname, *apiver, *freq, *step, *sr, *dvbs, *dvbs2;
	GtkWidget *tunfreq, *tunsr, *tunpol, *tunfec, *tunsys, *tunmod;
	GtkWidget *demvpid, *demapid, *demnpid, *demnip, *demnm, *demnmac;
	GSList *devices;
	gchar *vpid;
	GString *apids;
};
struct sdevwindow devwindow;

gint devwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
gint devwindow_dev_set(GtkWidget *widget, GdkEvent *event, gpointer data);
gint devwindow_dev_restart(GtkWidget *widget, GdkEvent *event, gpointer data);
void devwindow_dev_change(GtkComboBox *widget, gpointer data);


gint devwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	if (devwindow.vpid != NULL) {
		g_free(devwindow.vpid);
		devwindow.vpid = NULL;
	}
	
	if (devwindow.apids != NULL) {
		g_string_free(devwindow.apids, TRUE);
		devwindow.apids = NULL;
	}
	
	if (devwindow.devices != NULL) devwindow.devices = NULL;  
    
    devwindow.visible = FALSE;
	gtk_widget_destroy(devwindow.win);
    return FALSE;
}

gint devwindow_dev_set(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gint ind;
	
	if (devwindow.devices != NULL) {
		ind = gtk_combo_box_get_active(GTK_COMBO_BOX(devwindow.device));
		mainwindow_device_open(ind, FALSE);
	}
	
	devwindow_close(NULL, NULL, NULL);
    return FALSE;
}

gint devwindow_dev_restart(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gint ind;
	
	if (devwindow.devices != NULL) {
		ind = gtk_combo_box_get_active(GTK_COMBO_BOX(devwindow.device));
		mainwindow_device_open(ind, TRUE);
	}
	
	devwindow_close(NULL, NULL, NULL);
    return FALSE;
}

void devwindow_dev_change(GtkComboBox *widget, gpointer data)
{
	gint ind;
	struct _device *device;
	gchar devname[256];
		
	ind = gtk_combo_box_get_active(GTK_COMBO_BOX(devwindow.device));
	device = (struct _device *)g_slist_nth_data(devwindow.devices, ind);
	gtk_label_set_text(GTK_LABEL(devwindow.name), device->name);
	gtk_label_set_text(GTK_LABEL(devwindow.vendor), device->vendor);
	gtk_label_set_text(GTK_LABEL(devwindow.kdriver), device->driver);
	if (device->bus == DVBNAME_DEVICE_BUS_USB) {
		g_snprintf(devname, sizeof(devname), "USB: %s", device->slot);
	} else {
		g_snprintf(devname, sizeof(devname), "PCI: %s", device->slot);
	}
	gtk_label_set_text(GTK_LABEL(devwindow.slot), devname);
}


void devwindow_filters_foreach(gpointer data, gpointer user_data)
{
	struct _pid *currentpid = (struct _pid *)data;
		
	if (currentpid->type == PID_TYPE_VIDEO) {
		devwindow.vpid = g_strdup_printf("%u", currentpid->value);
	} else if (currentpid->type == PID_TYPE_AUDIO) {
		g_string_append_printf(devwindow.apids, "%u ", currentpid->value);
	}
}

gboolean devwindow_create(struct _frontend *frontend, struct _service *svc)
{
    GtkWidget *vbox, *hbox;
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *device_l, *name_l, *vendor_l, *kdriver_l, *slot_l;
	GtkWidget *sep;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *evbox;
	GtkWidget *chbox;
	GtkWidget *icon;
	gchar hex[7];
	gchar devname[256];
	GtkWidget *notebook;
	gint i = 0;
	gint ind = 0;
	struct _device *device;
	
	if (!devwindow.visible) {
	  devwindow.visible = TRUE;  
	  devwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  gtk_window_set_modal(GTK_WINDOW(devwindow.win), TRUE);
	  gtk_window_set_resizable(GTK_WINDOW(devwindow.win), FALSE);
	  devwindow.title = _("Device settings");
	  gtk_window_set_position(GTK_WINDOW(devwindow.win), GTK_WIN_POS_CENTER);
	  gtk_window_set_title(GTK_WINDOW(devwindow.win), devwindow.title);
	  gtk_widget_set_size_request(devwindow.win, 400, 300);
	  gtk_window_set_default_size(GTK_WINDOW(devwindow.win), 400, 300);
	  //gtk_widget_set_size_request(devwindow.win, 400, 300);
	  
	  vbox = gtk_vbox_new(FALSE, 0);
	  
	  fds = pango_font_description_from_string("14");
	  pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
	  devwindow.caption = gtk_label_new(_("Device settings"));
	  gtk_widget_modify_font(devwindow.caption, fds);
	  pango_font_description_free(fds);
	  evbox = gtk_event_box_new();
	  gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox),TRUE);
	  gdk_color_parse("white", &color);
	  gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
	  chbox = gtk_hbox_new(FALSE,0);
	  gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
	  icon = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO,GTK_ICON_SIZE_DIALOG);
	  gtk_box_pack_start(GTK_BOX(chbox),icon,FALSE,FALSE,15);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align),devwindow.caption);
	  gtk_box_pack_start(GTK_BOX(chbox),align,FALSE,FALSE,0);
	  gtk_container_add(GTK_CONTAINER(evbox),chbox);
	  
	  gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
	  
	  gentable = gtk_table_new(6,2,FALSE);
	  gtk_table_set_row_spacings(GTK_TABLE(gentable),2);
	  
	  device_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(device_l),_("<b>Device </b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), device_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,0,1);
	  
	  devwindow.device = gtk_combo_box_new_text();
	  
	  devwindow.devices = mainwindow_get_devices(NULL);
	  
	  if (devwindow.devices != NULL) {
		  for (i = 0; i<g_slist_length(devwindow.devices); i++) {
			  device = (struct _device *)g_slist_nth_data(devwindow.devices, i);
			  g_sprintf(devname, "Adapter %u : Frontend %u", device->adapter, device->frontend);
			  gtk_combo_box_append_text(GTK_COMBO_BOX(devwindow.device), devname);
			  if ((device->adapter == frontend->adapter_nr) && (device->frontend == frontend->frontend_nr)) ind = i;
		  }
	  } else {
		  gtk_combo_box_append_text(GTK_COMBO_BOX(devwindow.device),_("No devices found"));
		  gtk_widget_set_sensitive(GTK_WIDGET(devwindow.device), FALSE);
	  }
	  	  
	  gtk_combo_box_set_active(GTK_COMBO_BOX(devwindow.device), ind);
	  
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.device);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,0,1);
	  
	  if (devwindow.devices != NULL) device = (struct _device *)g_slist_nth_data(devwindow.devices, ind);
	  	  
	  name_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(name_l),_("<b>Device name:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), name_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	  
	  devwindow.name = gtk_label_new((devwindow.devices != NULL) ? device->name : "--");
	  gtk_label_set_max_width_chars(GTK_LABEL(devwindow.name),25);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.name),TRUE);
	  gtk_label_set_ellipsize(GTK_LABEL(devwindow.name), PANGO_ELLIPSIZE_END);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.name);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	  
	  vendor_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(vendor_l),_("<b>Device vendor:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), vendor_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,2,3);
	  
	  devwindow.vendor = gtk_label_new((devwindow.devices != NULL) ? device->vendor : "--");
	  gtk_label_set_max_width_chars(GTK_LABEL(devwindow.vendor),25);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.vendor),TRUE);
	  gtk_label_set_ellipsize(GTK_LABEL(devwindow.vendor), PANGO_ELLIPSIZE_END);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.vendor);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,2,3);

	  kdriver_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(kdriver_l),_("<b>Kernel driver:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), kdriver_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,3,4);
	  
	  devwindow.kdriver = gtk_label_new((devwindow.devices != NULL) ? device->driver : "--");
	  gtk_label_set_max_width_chars(GTK_LABEL(devwindow.kdriver),25);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.kdriver),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.kdriver);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,3,4);
	  
	  slot_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(slot_l),_("<b>Device bus:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), slot_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,4,5);
	  
	  if (devwindow.devices != NULL) {
		  if (device->bus == DVBNAME_DEVICE_BUS_USB) {
			  g_snprintf(devname, sizeof(devname), "USB: %s", device->slot);
		  } else {
			  g_snprintf(devname, sizeof(devname), "PCI: %s", device->slot);
		  }
	  }
	  devwindow.slot = gtk_label_new((devwindow.devices != NULL) ? devname : "--");
	  gtk_label_set_max_width_chars(GTK_LABEL(devwindow.slot),25);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.slot),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.slot);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,4,5);
	  
	  notebook = gtk_notebook_new();
	  GtkWidget *devcaption;
	  devcaption = gtk_label_new(_("Device"));
	  gtk_container_set_border_width(GTK_CONTAINER(gentable),10);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),gentable,devcaption);
	  
	  if (devwindow.devices != NULL) {
	  GtkWidget *drvtable = gtk_table_new(7,2,FALSE);
	  //gtk_table_set_row_spacings(GTK_TABLE(drvtable),5);
	  
	  GtkWidget *drvname_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(drvname_l),_("<b>Driver name:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), drvname_l);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,0,1,0,1);
	  
	  devwindow.drvname = gtk_label_new(frontend->driver);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.drvname),TRUE);
	  gtk_label_set_max_width_chars(GTK_LABEL(devwindow.drvname), 25);
	  gtk_label_set_ellipsize(GTK_LABEL(devwindow.drvname), PANGO_ELLIPSIZE_END);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.drvname);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,1,2,0,1);
	  
	  GtkWidget *apiver_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(apiver_l),_("<b>LinuxTV API:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), apiver_l);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,0,1,1,2);
	  
	  g_sprintf(devname,"%u.%u",((frontend->api_version & 0xFF00) >> 8),(frontend->api_version & 0x00FF));
	  devwindow.apiver = gtk_label_new(devname);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.apiver),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.apiver);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,1,2,1,2);
	  
	  GtkWidget *freq_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(freq_l),_("<b>Frequencies:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), freq_l);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,0,1,2,3);
	  
	  g_sprintf(devname,"%u - %u",frontend->frequency_min/1000, frontend->frequency_max/1000);
	  devwindow.freq = gtk_label_new(devname);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.freq),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.freq);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,1,2,2,3);
	  
	  GtkWidget *step_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(step_l),_("<b>Freq. stepsize:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), step_l);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,0,1,3,4);
	  
	  g_sprintf(devname,"%u", frontend->frequency_stepsize);
	  devwindow.step = gtk_label_new(devname);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.step),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.step);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,1,2,3,4);
	  
	  GtkWidget *sr_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(sr_l),_("<b>Symbolrates:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), sr_l);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,0,1,4,5);
	  
	  g_sprintf(devname,"%u - %u", frontend->symbol_rate_min/1000, frontend->symbol_rate_max/1000);
	  devwindow.sr = gtk_label_new(devname);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.sr),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.sr);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,1,2,4,5);
	  
	  GtkWidget *dvbs_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(dvbs_l),_("<b>DVB-S support:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), dvbs_l);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,0,1,5,6);
	  
	  devwindow.dvbs = gtk_label_new(frontend->dvbs_support ? _("Yes") : _("No"));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.dvbs),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.dvbs);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,1,2,5,6);
	  
	  GtkWidget *dvbs2_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(dvbs2_l),_("<b>DVB-S2 support:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), dvbs2_l);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,0,1,6,7);
	  
	  devwindow.dvbs2 = gtk_label_new(frontend->dvbs2_support ? _("Yes") : _("No"));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.dvbs2),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.dvbs2);
	  gtk_table_attach_defaults(GTK_TABLE(drvtable),align,1,2,6,7);

	  GtkWidget *drvcaption;
	  drvcaption = gtk_label_new(_("Current driver"));
	  gtk_container_set_border_width(GTK_CONTAINER(drvtable),10);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),drvtable,drvcaption);
	  
	  if ((frontend->tuned) || (frontend->open_mode_readonly)) {
	  GtkWidget *tuntable = gtk_table_new(6,2,FALSE);
	  
	  GtkWidget *tunfreq_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(tunfreq_l),_("<b>Frequency (IF):</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tunfreq_l);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,0,1,0,1);
	  
	  g_sprintf(devname,"%u",frontend->realfreq);
	  devwindow.tunfreq = gtk_label_new(devname);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.tunfreq),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.tunfreq);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,1,2,0,1);
	  
	  GtkWidget *tunsr_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(tunsr_l),_("<b>Symbolrate:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tunsr_l);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,0,1,1,2);
	  
	  g_sprintf(devname,"%u",frontend->realsr);
	  devwindow.tunsr = gtk_label_new(devname);
	  gtk_label_set_selectable(GTK_LABEL(devwindow.tunsr),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.tunsr);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,1,2,1,2);
	  
	  GtkWidget *tunpol_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(tunpol_l),_("<b>Polarisation:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tunpol_l);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,0,1,2,3);
	  
	  devwindow.tunpol = gtk_label_new(mainwindow_string_definition(DEFINITION_POLARISATION, svc->pol));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.tunpol),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.tunpol);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,1,2,2,3);
	  
	  GtkWidget *tunfec_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(tunfec_l),_("<b>FEC:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tunfec_l);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,0,1,3,4);
	  
	  devwindow.tunfec = gtk_label_new(mainwindow_string_definition(DEFINITION_FEC, frontend->realfec));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.tunfec),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.tunfec);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,1,2,3,4);
	  
	  GtkWidget *tunsys_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(tunsys_l),_("<b>System:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tunsys_l);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,0,1,4,5);
	  
	  devwindow.tunsys = gtk_label_new(mainwindow_string_definition(DEFINITION_SYSTEM, svc->sys));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.tunsys),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.tunsys);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,1,2,4,5);
	  
	  GtkWidget *tunmod_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(tunmod_l),_("<b>Modulation:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tunmod_l);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,0,1,5,6);
	  
	  devwindow.tunmod = gtk_label_new(mainwindow_string_definition(DEFINITION_MODULATION, svc->mod));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.tunmod),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.tunmod);
	  gtk_table_attach_defaults(GTK_TABLE(tuntable),align,1,2,5,6);

	  GtkWidget *tuncaption;
	  tuncaption = gtk_label_new(_("Tuner"));
	  gtk_container_set_border_width(GTK_CONTAINER(tuntable),10);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),tuntable,tuncaption);
      }
	  
	  if ((frontend->tuned) && (!frontend->open_mode_readonly)) {
	  
	  GtkWidget *demtable = gtk_table_new(6,2,FALSE);
	  //gtk_table_set_row_spacings(GTK_TABLE(tuntable),5);
	  
	  if (devwindow.vpid != NULL) {
		  g_free(devwindow.vpid);
		  devwindow.vpid = NULL;
	  }
	  
	  if (devwindow.apids != NULL) {
		  g_string_free(devwindow.apids, TRUE);
		  devwindow.apids = NULL;
	  }
	  
	  devwindow.apids = g_string_new("");
	  	  
	  g_slist_foreach(svc->pids, devwindow_filters_foreach, NULL);
	  	  
	  GtkWidget *demvpid_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(demvpid_l),_("<b>Video PID:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), demvpid_l);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,0,1,0,1);
	  
	  devwindow.demvpid = gtk_label_new((devwindow.vpid != NULL) ? devwindow.vpid : _("Disabled"));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.demvpid),TRUE);
	  gtk_label_set_max_width_chars(GTK_LABEL(devwindow.demvpid), 25);
	  gtk_label_set_ellipsize(GTK_LABEL(devwindow.demvpid), PANGO_ELLIPSIZE_END);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.demvpid);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,1,2,0,1);
	  
	  GtkWidget *demapid_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(demapid_l),_("<b>Audio PID:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), demapid_l);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,0,1,1,2);
	  
	  devwindow.demapid = gtk_label_new(((devwindow.apids->str != NULL) && (devwindow.apids->str[0] != '\0')) ? devwindow.apids->str : _("Disabled"));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.demapid),TRUE);
	  gtk_label_set_max_width_chars(GTK_LABEL(devwindow.demapid), 25);
	  gtk_label_set_ellipsize(GTK_LABEL(devwindow.demapid), PANGO_ELLIPSIZE_END);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.demapid);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,1,2,1,2);
	  
	  GtkWidget *demnpid_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(demnpid_l),_("<b>Network PID:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), demnpid_l);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,0,1,2,3);
	  
	  g_sprintf(devname,"%u", svc->npid);
	  devwindow.demnpid = gtk_label_new((svc->npid > 0) ? devname : _("Disabled"));
	  gtk_label_set_selectable(GTK_LABEL(devwindow.demnpid),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.demnpid);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,1,2,2,3);
	  
	  GtkWidget *demnip_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(demnip_l),_("<b>IP address:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), demnip_l);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,0,1,3,4);
	  
	  devwindow.demnip = gtk_label_new((svc->npid > 0) ? svc->nip : "--");
	  gtk_label_set_selectable(GTK_LABEL(devwindow.demnip),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.demnip);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,1,2,3,4);
	  
	  GtkWidget *demnm_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(demnm_l),_("<b>Netmask:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), demnm_l);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,0,1,4,5);
	  
	  devwindow.demnm = gtk_label_new((svc->npid > 0) ? svc->nmask : "--");
	  gtk_label_set_selectable(GTK_LABEL(devwindow.demnm),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.demnm);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,1,2,4,5);
	  
	  GtkWidget *demnmac_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(demnmac_l),_("<b>MAC address:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), demnmac_l);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,0,1,5,6);
	  
	  devwindow.demnmac = gtk_label_new((svc->npid > 0) ? svc->nmac : "--");
	  gtk_label_set_selectable(GTK_LABEL(devwindow.demnmac),TRUE);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), devwindow.demnmac);
	  gtk_table_attach_defaults(GTK_TABLE(demtable),align,1,2,5,6);

	  GtkWidget *demcaption;
	  demcaption = gtk_label_new(_("Demuxer"));
	  gtk_container_set_border_width(GTK_CONTAINER(demtable),10);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),demtable,demcaption);
      } //tuner.tuned
	  } //devices != NULL
	  
	  gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
	  
	  devwindow.restart = gtk_button_new_with_label(_("Restart"));
	  GtkWidget *image = gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON);
	  gtk_button_set_image(GTK_BUTTON(devwindow.restart), image);
	  devwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	  devwindow.apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
	  	  
	  hbox = gtk_hbox_new(FALSE, 0);
	  gtk_box_pack_end(GTK_BOX(hbox), devwindow.apply, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), devwindow.close, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), devwindow.restart, FALSE, FALSE, 5);
	  
	  if (g_slist_length(devwindow.devices) == 0) {
		  gtk_widget_set_sensitive(GTK_WIDGET(devwindow.apply), FALSE);
		  gtk_widget_set_sensitive(GTK_WIDGET(devwindow.restart), FALSE);
	  }
	  
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);

      g_signal_connect(G_OBJECT(devwindow.win), "destroy",
                       G_CALLBACK(devwindow_close), NULL);
	  
	  g_signal_connect(G_OBJECT(devwindow.close), "clicked",
                       G_CALLBACK(devwindow_close), NULL);
					   
	  g_signal_connect(G_OBJECT(devwindow.apply), "clicked",
                       G_CALLBACK(devwindow_dev_set), NULL);
					   
	  g_signal_connect(G_OBJECT(devwindow.restart), "clicked",
                       G_CALLBACK(devwindow_dev_restart), NULL);
					   
	  g_signal_connect(G_OBJECT(devwindow.device), "changed",
                       G_CALLBACK(devwindow_dev_change), NULL);

 	  gtk_container_add(GTK_CONTAINER(devwindow.win),vbox);
	  gtk_widget_show_all(devwindow.win);
	  
	  gtk_window_set_focus(GTK_WINDOW(devwindow.win), devwindow.apply);
	  
  } else {
	  
	  gtk_window_present(GTK_WINDOW(devwindow.win));
	  
  }
	  
      return TRUE;
}
