/*
 *      grstatus.c
 *      
 *      Copyright 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "grstatus.h"


static void gtk_grstatus_class_init(GtkGrstatusClass *klass);
static void gtk_grstatus_init(GtkGrstatus *grstatus);
static void gtk_grstatus_size_request(GtkWidget *widget, GtkRequisition *requisition);
static void gtk_grstatus_size_allocate(GtkWidget *widget, GtkAllocation *allocation);
static void gtk_grstatus_realize(GtkWidget *widget);
static gboolean gtk_grstatus_expose(GtkWidget *widget, GdkEventExpose *event);
static void gtk_grstatus_paint(GtkWidget *widget, GdkRectangle *area);
static void gtk_grstatus_destroy(GtkObject *object);

gint vsize;

GtkType gtk_grstatus_get_type(void)
{
	static GtkType gtk_grstatus_type = 0;
	
	if (!gtk_grstatus_type) {
		static const GtkTypeInfo gtk_grstatus_info = {
			"GtkGrstatus",
			sizeof(GtkGrstatus),
			sizeof(GtkGrstatusClass),
			(GtkClassInitFunc) gtk_grstatus_class_init,
			(GtkObjectInitFunc) gtk_grstatus_init,
			NULL,
			NULL,
			(GtkClassInitFunc) NULL
		};
		gtk_grstatus_type = gtk_type_unique(GTK_TYPE_WIDGET, &gtk_grstatus_info);
	}
	
	return gtk_grstatus_type;
}

void gtk_grstatus_set_value(GtkGrstatus *grstatus, gchar value)
{
	if (grstatus->ptr+1 > grstatus->valsize-1) grstatus->ptr = 0; else grstatus->ptr++;
	grstatus->values[grstatus->ptr] = value;
	gtk_widget_queue_draw(GTK_WIDGET(grstatus));
}

void gtk_grstatus_set_colors(GtkGrstatus *grstatus, GdkColor *mincolor, GdkColor *maxcolor)
{
	grstatus->mincolor = *mincolor;
	grstatus->maxcolor = *maxcolor;
}

GtkWidget *gtk_grstatus_new(gint valsize)
{
	vsize = valsize;
	return GTK_WIDGET(gtk_type_new(gtk_grstatus_get_type()));
}


static void gtk_grstatus_class_init(GtkGrstatusClass *klass)
{
	GtkWidgetClass *widget_class;
	GtkObjectClass *object_class;
	
	widget_class = (GtkWidgetClass *)klass;
	object_class = (GtkObjectClass *)klass;
	
	widget_class->realize = gtk_grstatus_realize;
	widget_class->size_request = gtk_grstatus_size_request;
	widget_class->size_allocate = gtk_grstatus_size_allocate;
	widget_class->expose_event = gtk_grstatus_expose;
	
	object_class->destroy = gtk_grstatus_destroy;
}


static void gtk_grstatus_init(GtkGrstatus *grstatus)
{
	gint i;
	
	grstatus->valsize = vsize;
	grstatus->values = (gchar *)g_malloc(grstatus->valsize);
	for (i=0; i<grstatus->valsize; i++) grstatus->values[i] = 0; 
	grstatus->ptr = 0;
	gdk_color_parse("white", &grstatus->mincolor);
	gdk_color_parse("green3", &grstatus->maxcolor); 
}


static void gtk_grstatus_size_request(GtkWidget *widget, GtkRequisition *requisition)
{
	g_return_if_fail(widget != NULL);
	g_return_if_fail(GTK_IS_GRSTATUS(widget));
	g_return_if_fail(requisition != NULL);
	
	requisition->width = GTK_GRSTATUS(widget)->valsize;
	requisition->height = 20;
}


static void gtk_grstatus_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
{
	g_return_if_fail(widget != NULL);
	g_return_if_fail(GTK_IS_GRSTATUS(widget));
	g_return_if_fail(allocation != NULL);
	
	widget->allocation = *allocation;
	
	if (GTK_WIDGET_REALIZED(widget)) {
		gdk_window_move_resize(widget->window, allocation->x, allocation->y, allocation->width, allocation->height);
	}
}


static void gtk_grstatus_realize(GtkWidget *widget)
{
	GdkWindowAttr attributes;
	guint attributes_mask;
	
	g_return_if_fail(widget != NULL);
	g_return_if_fail(GTK_IS_GRSTATUS(widget));
	
	GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);
	
	attributes.window_type = GDK_WINDOW_CHILD;
	attributes.x = widget->allocation.x;
	attributes.y = widget->allocation.y;
	attributes.width = GTK_GRSTATUS(widget)->valsize;
	attributes.height = 20;
	
	attributes.wclass = GDK_INPUT_OUTPUT;
	attributes.event_mask = gtk_widget_get_events(widget) | GDK_EXPOSURE_MASK;
	
	attributes_mask = GDK_WA_X | GDK_WA_Y;
	
	widget->window = gdk_window_new(gtk_widget_get_parent_window(widget), &attributes, attributes_mask);
	gdk_window_set_user_data(widget->window, widget);
	
	widget->style = gtk_style_attach(widget->style, widget->window);
	gtk_style_set_background(widget->style, widget->window, GTK_STATE_NORMAL);
}


static gboolean gtk_grstatus_expose(GtkWidget *widget, GdkEventExpose *event)
{
	g_return_val_if_fail(widget != NULL, FALSE);
	g_return_val_if_fail(GTK_IS_GRSTATUS(widget), FALSE);
	g_return_val_if_fail(event != NULL, FALSE);
	
	gtk_grstatus_paint(widget, &(event->area));
	
	return FALSE;
}


static void gtk_grstatus_paint(GtkWidget *widget, GdkRectangle *area)
{
	cairo_t *cr;
	gint i;
	gdouble peakvalue;
	gdouble minr, ming, minb;
	gdouble slr, slg, slb;
	
	cr = gdk_cairo_create(widget->window);
	
	cairo_translate(cr, 0, 20);
	
	cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
	cairo_paint(cr);
	
	minr = GTK_GRSTATUS(widget)->mincolor.red/65535.0;
	slr = (GTK_GRSTATUS(widget)->maxcolor.red - GTK_GRSTATUS(widget)->mincolor.red)/65535.0;
	ming = GTK_GRSTATUS(widget)->mincolor.green/65535.0;
	slg = (GTK_GRSTATUS(widget)->maxcolor.green - GTK_GRSTATUS(widget)->mincolor.green)/65535.0;
	minb = GTK_GRSTATUS(widget)->mincolor.blue/65535.0;
	slb = (GTK_GRSTATUS(widget)->maxcolor.blue - GTK_GRSTATUS(widget)->mincolor.blue)/65535.0;
	
	for (i=0; i<=GTK_GRSTATUS(widget)->valsize-1; i++) {
		peakvalue = (GTK_GRSTATUS(widget)->values[GTK_GRSTATUS(widget)->ptr])/(100.0/18.0);
		cairo_set_source_rgb(cr, minr+(slr*peakvalue/18.0), ming+(slg*peakvalue/18.0), minb+(slb*peakvalue/18.0));
		cairo_rectangle(cr, (gdouble)i, 0.0, 1.0, -peakvalue);
		cairo_fill(cr);
		if (GTK_GRSTATUS(widget)->ptr+1 > GTK_GRSTATUS(widget)->valsize-1) GTK_GRSTATUS(widget)->ptr = 0; else GTK_GRSTATUS(widget)->ptr++;
	}
	cairo_destroy(cr);
	
	gtk_paint_shadow(widget->style, widget->window, GTK_STATE_NORMAL, GTK_SHADOW_IN, area, widget,"", 0, 0, widget->allocation.width, widget->allocation.height);
}


static void gtk_grstatus_destroy(GtkObject *object)
{
	GtkGrstatus *grstatus;
	GtkGrstatusClass *klass;
	
	g_return_if_fail(object != NULL);
	g_return_if_fail(GTK_IS_GRSTATUS(object));
	
	grstatus = GTK_GRSTATUS(object);
	
	klass = gtk_type_class(gtk_widget_get_type());
	
	if (GTK_OBJECT_CLASS(klass)->destroy) {
		(* GTK_OBJECT_CLASS(klass)->destroy) (object);
	}
}

