/*
 *      streamserver.c
 *      
 *      Copyright 2009-2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/select.h>
#include <sys/poll.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>
#include <limits.h>
#include <sys/resource.h>
#include <sys/time.h>
#include <sys/utsname.h>

#ifdef linux
	#include <sys/prctl.h>
	#include <linux/prctl.h>
#endif

#include "csadec.h"
#include "saferwlocks.h"
#include "signalshed.h"
#include "streamserver.h"
#include "svrsettings.h"
#include "../common/serialize.h"
#include "../resources.h"


#ifndef INFTIM
	#define INFTIM -1
#endif

union semun {
	int val;
	struct semid_ds *buf;
	unsigned short *array;
};

enum {
	FDTRANSPORT_START_RECORD = 0,
	FDTRANSPORT_STOP_RECORD,
	FDTRANSPORT_CHANGE_DEVICE,
	FDTRANSPORT_NEW_TCP_CLIENT
};

//Prototypes
static int add_client(struct sclient *client);
static int remove_client(pid_t pid);
static void streamserver_cleanup_client_slots(void);
static int transmission_process_send(int *index, unsigned int *errorcount, unsigned int *idlecount);
static void transmission_process_signal_handler(int sig, siginfo_t *si, void *unused);
static void transmission_process(struct sclient *client);
static int streamserver_find_pid(unsigned short pid);
static void streamserver_tcp_listen_process_signal_handler(int sig, siginfo_t *si, void *unused);
static void main_signal_handler(int sig, siginfo_t *si, void *unused);
static void start_daemon(void);


rwlockshed_t rwlockshed;
signalshed_t signalshed;

int shmid, clisemid;
struct memblock *procmblock;
char readconfflag = 0, readsockflag = 0, cleanupflag = 0, exitflag = 0;
int fdtransport[2];
char tmexitflag = 0;
char tlpexitflag = 0;
char rpexitflag = 0;


static int add_client(struct sclient *client)
{
	int i;
	int resindex;
	char abuf[64];
	char processname[16];
	
	struct sembuf semoperations[1];
	
	resindex = -1;
	memset(processname, 0, sizeof(processname));
	
	semoperations[0].sem_num = 0;
	semoperations[0].sem_op = -1;
	semoperations[0].sem_flg = 0;
	
	if (semop(clisemid, semoperations, 1) == -1) {
		perror("[-] Add client semop call");
	}

	
	switch (client->type) {
		case CLIENT_TYPE_READ_HELPER:
			procmblock->clients[READ_PROCESS_INDEX].type = client->type;
			procmblock->clients[READ_PROCESS_INDEX].family = client->family;
			procmblock->clients[READ_PROCESS_INDEX].pid = client->pid;
			procmblock->clients[READ_PROCESS_INDEX].socketfd = client->socketfd;
			procmblock->clients[READ_PROCESS_INDEX].pidfilter = 0;
			procmblock->clients[READ_PROCESS_INDEX].transmittedbytes = 0;
			procmblock->clients[READ_PROCESS_INDEX].ready = 0;
			procmblock->clients[READ_PROCESS_INDEX].starttime = time(NULL);
			if (client->useragent[0] != '\0') strncpy((char *)&procmblock->clients[READ_PROCESS_INDEX].useragent, client->useragent, strlen(client->useragent)); else memset(&procmblock->clients[READ_PROCESS_INDEX].useragent, 0, sizeof(procmblock->clients[READ_PROCESS_INDEX].useragent));
			memset(procmblock->clients[READ_PROCESS_INDEX].ifname, 0, sizeof(procmblock->clients[LISTEN_PROCESS_INDEX].ifname));
			snprintf(processname, sizeof(processname), "gsfstrsvr/read");
			printf("[+] READ PROCESS started (ID: %i/%i)\n", client->pid, READ_PROCESS_INDEX);
			resindex = READ_PROCESS_INDEX;
			break;
		
		case CLIENT_TYPE_TCP_LISTEN_HELPER:
			procmblock->clients[LISTEN_PROCESS_INDEX].type = client->type;
			procmblock->clients[LISTEN_PROCESS_INDEX].family = client->family;
			procmblock->clients[LISTEN_PROCESS_INDEX].pid = client->pid;
			procmblock->clients[LISTEN_PROCESS_INDEX].socketfd = client->socketfd;
			procmblock->clients[LISTEN_PROCESS_INDEX].pidfilter = 0;
			procmblock->clients[LISTEN_PROCESS_INDEX].transmittedbytes = 0;
			procmblock->clients[LISTEN_PROCESS_INDEX].ready = 0;
			procmblock->clients[LISTEN_PROCESS_INDEX].starttime = time(NULL);
			if (client->useragent[0] != '\0') strncpy((char *)&procmblock->clients[LISTEN_PROCESS_INDEX].useragent, client->useragent, strlen(client->useragent)); else memset(&procmblock->clients[LISTEN_PROCESS_INDEX].useragent, 0, sizeof(procmblock->clients[LISTEN_PROCESS_INDEX].useragent));
			memset(procmblock->clients[LISTEN_PROCESS_INDEX].ifname, 0, sizeof(procmblock->clients[LISTEN_PROCESS_INDEX].ifname));
			snprintf(processname, sizeof(processname), "gsfstrsvr/tcplp");
			printf("[+] TCP LISTEN PROCESS started at port %d (ID: %i/%i)\n", ntohs(client->address.sin_port), client->pid, LISTEN_PROCESS_INDEX);
			resindex = LISTEN_PROCESS_INDEX;
			break;
		
		
		case CLIENT_TYPE_UDP_BROADCAST:
			procmblock->clients[UDP_BC_PROCESS_INDEX].type = client->type;
			procmblock->clients[UDP_BC_PROCESS_INDEX].family = client->family;
			procmblock->clients[UDP_BC_PROCESS_INDEX].pid = client->pid;
			procmblock->clients[UDP_BC_PROCESS_INDEX].socketfd = client->socketfd;
			procmblock->clients[UDP_BC_PROCESS_INDEX].pidfilter = 0;
			procmblock->clients[UDP_BC_PROCESS_INDEX].transmittedbytes = 0;
			procmblock->clients[UDP_BC_PROCESS_INDEX].ready = 0;
			procmblock->clients[UDP_BC_PROCESS_INDEX].starttime = time(NULL);
			procmblock->clients[UDP_BC_PROCESS_INDEX].family = CLIENT_FAMILY_V4_TRANSPORT;
			if (client->useragent[0] != '\0') strncpy((char *)&procmblock->clients[UDP_BC_PROCESS_INDEX].useragent, client->useragent, strlen(client->useragent)); else memset(&procmblock->clients[UDP_BC_PROCESS_INDEX].useragent, 0, sizeof(procmblock->clients[UDP_BC_PROCESS_INDEX].useragent));
			memset(procmblock->clients[UDP_BC_PROCESS_INDEX].ifname, 0, sizeof(procmblock->clients[UDP_BC_PROCESS_INDEX].ifname));
			procmblock->numclients++;
			close(fdtransport[1]);
			snprintf(processname, sizeof(processname), "gsfstrsvr/udpbc");
			printf("[+] UDP BROADCAST started %s:%d (ID: %i/%i)\n", inet_ntop(AF_INET, (struct in_addr *)&(client->address.sin_addr), abuf, sizeof(abuf)), ntohs(client->address.sin_port), client->pid, UDP_BC_PROCESS_INDEX);
			resindex = UDP_BC_PROCESS_INDEX;
			break;
		
		
		case CLIENT_TYPE_UDP_MULTICAST:
			procmblock->clients[UDP_MC_PROCESS_INDEX].type = client->type;
			procmblock->clients[UDP_MC_PROCESS_INDEX].family = client->family;
			procmblock->clients[UDP_MC_PROCESS_INDEX].pid = client->pid;
			procmblock->clients[UDP_MC_PROCESS_INDEX].socketfd = client->socketfd;
			procmblock->clients[UDP_MC_PROCESS_INDEX].pidfilter = 0;
			procmblock->clients[UDP_MC_PROCESS_INDEX].transmittedbytes = 0;
			procmblock->clients[UDP_MC_PROCESS_INDEX].ready = 0;
			procmblock->clients[UDP_MC_PROCESS_INDEX].starttime = time(NULL);
			procmblock->clients[UDP_MC_PROCESS_INDEX].family = CLIENT_FAMILY_V4_TRANSPORT;
			if (client->useragent[0] != '\0') strncpy((char *)&procmblock->clients[UDP_MC_PROCESS_INDEX].useragent, client->useragent, strlen(client->useragent)); else memset(&procmblock->clients[UDP_MC_PROCESS_INDEX].useragent, 0, sizeof(procmblock->clients[UDP_MC_PROCESS_INDEX].useragent));
			memset(procmblock->clients[UDP_MC_PROCESS_INDEX].ifname, 0, sizeof(procmblock->clients[UDP_MC_PROCESS_INDEX].ifname));
			procmblock->numclients++;
			close(fdtransport[1]);
			snprintf(processname, sizeof(processname), "gsfstrsvr/udpmc");
			printf("[+] UDP MULTICAST started %s:%d (ID: %i/%i)\n", inet_ntop(AF_INET, (struct in_addr *)&(client->address.sin_addr), abuf, sizeof(abuf)), ntohs(client->address.sin_port), client->pid, UDP_MC_PROCESS_INDEX);
			resindex = UDP_MC_PROCESS_INDEX;
			break;
		
		
		case CLIENT_TYPE_TCP_UNICAST:
			for (i=TCP_FIRST_CLIENT; i<MAXCLIENTS; i++) {
				if (procmblock->clients[i].pid == 0) {
					procmblock->clients[i].type = client->type;
					procmblock->clients[i].family = client->family;
					procmblock->clients[i].pid = client->pid;
					procmblock->clients[i].socketfd = client->socketfd;
					procmblock->clients[i].ready = 0;
					procmblock->clients[i].starttime = time(NULL);
					procmblock->clients[i].family = CLIENT_FAMILY_V6_TRANSPORT;
					procmblock->clients[i].transmittedbytes = client->transmittedbytes;
					memcpy(&procmblock->clients[i].addressv6, &client->addressv6, sizeof(struct sockaddr_in6));
					if (client->useragent[0] != '\0') strncpy((char *)&procmblock->clients[i].useragent, client->useragent, strlen(client->useragent)); else memset(&procmblock->clients[i].useragent, 0, sizeof(procmblock->clients[i].useragent));
					if ((client->pidfilter >= 32) && (client->pidfilter <= 8191)) procmblock->clients[i].pidfilter = client->pidfilter; else procmblock->clients[i].pidfilter = 0;
					procmblock->numclients++;
					procmblock->numtcpclients++;
					close(fdtransport[1]);
					snprintf(processname, sizeof(processname), "gsfstrsvr/tcpuc");
					printf("[+] TCP UNICAST started with client %s:%d (ID: %i/%i)\n", inet_ntop(AF_INET6, (struct in_addr6 *)&(client->addressv6.sin6_addr), abuf, sizeof(abuf)), ntohs(client->addressv6.sin6_port), client->pid, i);
					resindex = i;
					break;
				}
			}
			break;
		
		default: 
			resindex = -1;
			break;
	}
	
	semoperations[0].sem_num = 0;
	semoperations[0].sem_op =  1;
	semoperations[0].sem_flg = 0;
	
	if (semop(clisemid, semoperations, 1) == -1) {
		perror("[-] Add client semop call");
	}
	
	#ifdef linux
		if (resindex != -1) prctl(PR_SET_NAME, (char *)&processname, 0, 0, 0);
	#endif
	
	if ((procmblock->sheduler == SHEDULER_SIGNAL) && (resindex != -1) && (resindex != LISTEN_PROCESS_INDEX)) 
		signalshed_add_client(signalshed, client->pid, resindex);
	
	return resindex;
}

static int remove_client(pid_t pid)
{
	int i;
	char abuf[64];
	
	struct sembuf semoperations[1];
	
	for (i=0; i<MAXCLIENTS; i++) {
		if (procmblock->clients[i].pid == pid) {
			
			semoperations[0].sem_num = 0;
			semoperations[0].sem_op = -1;
			semoperations[0].sem_flg = 0;
			
			if (semop(clisemid, semoperations, 1) == -1) {
				perror("[-] Remove client semop call");
			}
			
			switch (i) {
				case READ_PROCESS_INDEX: {
					printf("[+] READ PROCESS stopped (ID: %i/%i)\n", procmblock->clients[i].pid, i);
					break;
				}
				
				case LISTEN_PROCESS_INDEX: {
					printf("[+] TCP LISTEN PROCESS stopped at port %d (ID: %i/%i)\n", ntohs(procmblock->clients[i].address.sin_port), procmblock->clients[i].pid, i);
					break;
				}
				
				case UDP_BC_PROCESS_INDEX: {
					printf("[+] UDP BROADCAST stopped %s:%d (ID: %i/%i)\n", inet_ntop(AF_INET, (struct in_addr *)&(procmblock->clients[i].address.sin_addr), abuf, sizeof(abuf)), ntohs(procmblock->clients[i].address.sin_port), procmblock->clients[i].pid, i);
					break;
				}
				
				case UDP_MC_PROCESS_INDEX: {
					printf("[+] UDP MULTICAST stopped %s:%d (ID: %i/%i)\n", inet_ntop(AF_INET, (struct in_addr *)&(procmblock->clients[i].address.sin_addr), abuf, sizeof(abuf)), ntohs(procmblock->clients[i].address.sin_port), procmblock->clients[i].pid, i);
					break;				
				}
				
				default: {
					printf("[+] TCP UNICAST connection closed with client %s:%d (ID: %i/%i)\n", inet_ntop(AF_INET6, (struct in_addr6 *)&(procmblock->clients[i].addressv6.sin6_addr), abuf, sizeof(abuf)), ntohs(procmblock->clients[i].addressv6.sin6_port), procmblock->clients[i].pid, i);
					memset(&procmblock->clients[i].addressv6, 0, sizeof(struct sockaddr_in6));
					procmblock->numtcpclients--;
					break;
				}
			}
			
			//procmblock->clients[i].pid = 0;
			procmblock->clients[i].family = CLIENT_FAMILY_STOPPED;
			procmblock->clients[i].pidfilter = 0;
			procmblock->clients[i].socketfd = 0;
			//procmblock->clients[i].starttime = 0;
			procmblock->clients[i].transmittedbytes = 0;
			memset(&procmblock->clients[i].useragent, 0, sizeof(procmblock->clients[i].useragent));
			
			if ((i > 0) && (i != LISTEN_PROCESS_INDEX)) {
				procmblock->numclients--;
				procmblock->readyclients--;
			}
			
			semoperations[0].sem_num = 0;
			semoperations[0].sem_op =  1;
			semoperations[0].sem_flg = 0;
			
			if (semop(clisemid, semoperations, 1) == -1) {
				perror("[-] Remove client semop call");
			}
			
			if (procmblock->sheduler == SHEDULER_SIGNAL) signalshed_remove_client(signalshed, pid, i);
			
			return i;
		}
	}
	
	return -1;
}

static void streamserver_cleanup_client_slots(void)
{
	int i, status;
	//struct sembuf semoperations[1];
	
	for (i=0; i<MAXCLIENTS; i++) {
		if ((procmblock->clients[i].pid > 0) && ((procmblock->clients[i].family == CLIENT_FAMILY_STOPPED) || (kill(procmblock->clients[i].pid, 0) != 0))) {
			if (kill(procmblock->clients[i].pid, 0) != 0) {
				remove_client(procmblock->clients[i].pid);
			}
			if (waitpid(procmblock->clients[i].pid, &status, WNOHANG) != -1) {
				printf("[+] Freed resource slot of process (ID: %i/%i)\n", procmblock->clients[i].pid, i);
				procmblock->clients[i].pid = 0;
				procmblock->clients[i].starttime = 0;
			}
		}
	}
}

unsigned int streamserver_get_free_tcp_client_slots(void)
{
	int i;
	unsigned int count = 0;
	//struct sembuf semoperations[1];
	
	for (i=TCP_FIRST_CLIENT; i<MAXCLIENTS; i++) {
		if (procmblock->clients[i].pid == 0) {
			count++;
		}
	}
	
	return count;
}

static int transmission_process_send(int *index, unsigned int *errorcount, unsigned int *idlecount)
{
	int socketfd;
	int bufptr;
	unsigned char recvbuffer[128];
	ssize_t movedbytes;
	int errorcode;
	
	unsigned short pid;
	unsigned char adfield;
	unsigned char counter;
	unsigned int adfield_length;
	unsigned char pidbuffer[BUFSIZE];
	unsigned int pidbuffersize;
	unsigned int bytes;
	
	struct sembuf semoperations[1];
	
	
	if (procmblock->clients[*index].pid == getpid()) {
		errno = 0;
		if (recv(procmblock->clients[*index].socketfd, recvbuffer, sizeof(recvbuffer), 0) <= 0) {
			if (errno != EWOULDBLOCK) {
				tmexitflag = 1;
				goto finalizeclient;
			}
		}
		
		bufptr = 0;
		
		switch(*index) {
			case LISTEN_PROCESS_INDEX: {
				goto finalizeclient;
				break;
			}
			
			case UDP_BC_PROCESS_INDEX: {
				do {
					if ((movedbytes = sendto(procmblock->clients[*index].socketfd, &procmblock->buffer[bufptr], SENDSIZE, 0, (struct sockaddr *)&procmblock->clients[*index].address, sizeof(struct sockaddr))) <= 0) {
						*errorcount++;
						errorcode = errno;
						perror("[-] UDP BROADCAST error while sendto call");
						goto finalizeclient;
					}
					bufptr = bufptr + SENDSIZE;
				} while (bufptr != BUFSIZE);
				break;				
			}
			
			case UDP_MC_PROCESS_INDEX: {
				do {
					if ((movedbytes = sendto(procmblock->clients[*index].socketfd, &procmblock->buffer[bufptr], SENDSIZE, 0, (struct sockaddr *)&procmblock->clients[*index].address, sizeof(struct sockaddr))) <= 0) {
						*errorcount++;
						errorcode = errno;
						perror("[-] UDP MULTICAST error while sendto call");
						goto finalizeclient;
					}
					bufptr = bufptr + SENDSIZE;
				} while (bufptr != BUFSIZE);
				break;
			}
			
			default: {
				if (procmblock->clients[*index].pidfilter != 0) {
					memset(pidbuffer, 0, sizeof(pidbuffer));
					pidbuffersize = 0;
					while ((bufptr < BUFSIZE-1) && (bufptr != -1)) {
						pid = (((procmblock->buffer[bufptr+1] & 0x1f) << 8) | procmblock->buffer[bufptr+2]);
						adfield = (procmblock->buffer[bufptr+3] & 0x30) >> 4;
						counter = (procmblock->buffer[bufptr+3] & 0x0f);
						if (adfield == 0x3) adfield_length = procmblock->buffer[bufptr+4]; else adfield_length = 0;
						
						if (pid == procmblock->clients[*index].pidfilter) {
							memcpy(&pidbuffer[pidbuffersize], &procmblock->buffer[bufptr+4+adfield_length+1], 188-4-adfield_length-1);
							pidbuffersize = pidbuffersize + 188 - 4 - adfield_length - 1;
						}
						
						bufptr = bufptr + 188;
						if (procmblock->buffer[bufptr] != 0x47) {
							while (bufptr < BUFSIZE) {
								if (procmblock->buffer[bufptr] != 0x47) break;
								bufptr++;
							}
							if (bufptr == BUFSIZE) bufptr = -1;
						}
					}
					
					if (pidbuffersize > 0) {
						bufptr = 0;
						while (bufptr < pidbuffersize) {
							if (pidbuffersize - bufptr > SENDSIZE) bytes = SENDSIZE; else bytes = pidbuffersize - bufptr;
							if ((movedbytes = send(procmblock->clients[*index].socketfd, &pidbuffer[bufptr], bytes, 0)) <= 0) {
								if (errno != EWOULDBLOCK) {
									*errorcount++;
									errorcode = errno;
									perror("[-] TCP UNICAST error while send call");
									goto finalizeclient;
								}
							}
							bufptr = bufptr + bytes;
						}
						break;
					} else break;
				} else {
					do {
						if ((movedbytes = send(procmblock->clients[*index].socketfd, &procmblock->buffer[bufptr], SENDSIZE, 0)) <= 0) {
							if (errno != EWOULDBLOCK) {
								*errorcount++;
								errorcode = errno;
								perror("[-] TCP UNICAST error while send call");
								goto finalizeclient;
							}
						}
						bufptr = bufptr + SENDSIZE;
					} while (bufptr != BUFSIZE);
				}
			}
		}
		
		procmblock->clients[*index].transmittedbytes += movedbytes;
		procmblock->clients[*index].ready = 1;
	}
		
	finalizeclient:
	
	semoperations[0].sem_num = 1;
	semoperations[0].sem_op = -1;
	semoperations[0].sem_flg = 0;
	
	if (semop(clisemid, semoperations, 1) == -1) {
		perror("[-] Client semop call");
	}
	
	procmblock->readyclients++;
	
	semoperations[0].sem_num = 1;
	semoperations[0].sem_op =  1;
	semoperations[0].sem_flg = 0;
	
	if (semop(clisemid, semoperations, 1) == -1) {
		perror("[-] Client semop call");
	}
	
	if (movedbytes > 0) {
		*idlecount = 0;
	} else if (errorcode != EWOULDBLOCK) {
		*idlecount++;
	}
	
	if ((procmblock->sheduler == SHEDULER_SIGNAL) && (procmblock->readyclients >= procmblock->numclients)) {
		signalshed_post(signalshed);
	}
}

static void transmission_process_signal_handler(int sig, siginfo_t *si, void *unused)
{
	if ((sig == SIGINT) || (sig == SIGTERM)) {
		tmexitflag = 1;
	}
}

static void transmission_process(struct sclient *client)
{
	struct sigaction sa;
	int flagsval;
	int index;
	unsigned int errorcounter;
	unsigned int idlecounter;
	int dsignal = SIGTERM;
	
	client->pid = getpid();
	
	errorcounter = 0;
	idlecounter = 0;
	
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	sa.sa_sigaction = transmission_process_signal_handler;
		
	if (sigaction(SIGCONT, &sa, NULL) == -1) {
		printf("[-] Error calling SIGCONT handler procedure in client process\n");
	}
			
	if (sigaction(SIGINT, &sa, NULL) == -1) {
		printf("[-] Error calling SIGINT handler procedure in client process\n");
	}
	
	if (sigaction(SIGTERM, &sa, NULL) == -1) {
		printf("[-] Error calling SIGTERM handler procedure in client process\n");
	}
	
	#ifdef linux
		prctl(PR_SET_PDEATHSIG, dsignal, 0, 0, 0);
	#endif
	
	flagsval = fcntl(client->socketfd, F_GETFL, 0);
	fcntl(client->socketfd, F_SETFL, flagsval | O_NONBLOCK);
	
	if ((index = add_client(client)) == -1) {
		printf("[-] Failed to add client info into storage\n");
		close(client->socketfd);
		exit(EXIT_FAILURE);
	}
	
	while (!tmexitflag) {
		if (procmblock->sheduler == SHEDULER_SIGNAL) {
			tmexitflag |= signalshed_main(signalshed, &index, &errorcounter, &idlecounter, transmission_process_send);
		} else if (procmblock->sheduler == SHEDULER_RWLOCK) {
			rwlockshed_readlock(rwlockshed);
			transmission_process_send(&index, &errorcounter, &idlecounter);
			rwlockshed_readunlock(rwlockshed);
		}
		
		if (errorcounter >= ERROR_COUNT_LIMIT) {
			printf("[-] Unable to establish connection with client (ID: %i/%i)\n", client->pid, index);
			tmexitflag = 1;
		}
		
		if (idlecounter >= IDLE_COUNT_LIMIT) {
			printf("[-] Client not recieving transmitted data (ID: %i/%i)\n", client->pid, index);
			tmexitflag = 1;
		}
	}
 	
	close(client->socketfd);
	
	if (remove_client(client->pid) == -1) {
		printf("[-] No client removed - seems like internal error\n");
	}
	
	//printf("-->EXIT %i\n", client->pid);
	
	exit(EXIT_SUCCESS);
}

static int parse_request(char *request, char *buffer, size_t size, char *start, char *end)
{
	char *first, *second;
	int numsym;
	
	memset(buffer, 0, size);
	if ((first = strstr(request, start)) == NULL) return -1;
	if ((second = strstr(first+1, end)) == NULL) return -1;
	
	if ((numsym = second - first - strlen(start)) <= 0) return 0;
	if (numsym >= size - 1) numsym = size - 1;
	
	strncpy(buffer, first + strlen(start), numsym);
	buffer[numsym] = '\0';

	return numsym;
}

int streamserver_process_exists(pid_t pid)
{
	if (pid == 0) return 0;
	
	if (kill(pid, 0) == 0) {
		return 1;
	} else {
		return 0;
	}
}

static int streamserver_find_pid(unsigned short pid)
{
	int i, pidfound;
	
	pidfound = 0;
	for (i=0; i<256; i++) {
		if (procmblock->streampids[i].value == 0) break;
		if (procmblock->streampids[i].value == pid) {
			if (procmblock->streampids[i].enabled) {
				pidfound = 1;
				break;
			}
		}
	}
	
	return pidfound;
}

void streamserver_stop_client(unsigned int clientid)
{
	if ((clientid < TCP_FIRST_CLIENT) || (clientid > MAXCLIENTS)) return;
	
	if (!streamserver_process_exists(procmblock->clients[clientid].pid)) {
		printf("[-] Unable to stop client - not running\n");
		return;
	}
	
	kill(procmblock->clients[clientid].pid, SIGTERM);
}

void streamserver_tcp_listen_process_signal_handler(int sig, siginfo_t *si, void *unused)
{
	if ((sig == SIGINT) || (sig == SIGTERM)) {
		tlpexitflag = 1;
	}
}

void streamserver_init_tcp_listen_process(void)
{
	struct sigaction sa;
	sigset_t blockmask, oldmask;
	const int sflag = 1;
	int helperindex;
	int dsignal = SIGTERM;
	int listenfdv6;
	struct pollfd sockets[1];
	int readysockets;
	socklen_t addressv6len = sizeof(struct sockaddr_in6);
	char recvbuffer[4096];
	struct sclient helperclient;
	pid_t pid;
	struct sclient client;
	struct msghdr msg;
	char ccmsg[CMSG_SPACE(sizeof(int))];
	struct cmsghdr *cmsg;
	struct iovec vec[2];
	char cmdbuf[1] = {FDTRANSPORT_NEW_TCP_CLIENT};
	
	
	if ((pid = fork()) < 0) {
		//error
		printf("[-] Error in fork system call\n");
		exit(EXIT_FAILURE);
	} else if (pid == 0) {

		sa.sa_flags = SA_SIGINFO;
		sigemptyset(&sa.sa_mask);
		sa.sa_sigaction = streamserver_tcp_listen_process_signal_handler;
		
		if (sigaction(SIGTERM, &sa, NULL) == -1) {
			printf("[-] Error calling SIGCHLD handler procedure in listen thread\n");
		}
		
		if (sigaction(SIGINT, &sa, NULL) == -1) {
			printf("[-] Error calling SIGINT handler procedure in listen thread\n");
		}
		
		sigemptyset(&blockmask);
		sigaddset(&blockmask, SIGCONT);
		sigprocmask(SIG_BLOCK, &blockmask, &oldmask);
		
		#ifdef linux
			prctl(PR_SET_PDEATHSIG, dsignal, 0, 0, 0);
		#endif
				
		if ((listenfdv6 = socket(AF_INET6, SOCK_STREAM, 0)) == -1) {
			printf("[-] Failed to create listen socket for incoming TCP connections: %s\n", strerror(errno));
			exit(EXIT_FAILURE);
		}
			
		if (setsockopt(listenfdv6, SOL_SOCKET, SO_REUSEADDR, &sflag, sizeof(sflag)) < 0) {
			printf("[-] Failed to set socket flags: %s\n", strerror(errno));
			exit(EXIT_FAILURE);
		}
		
		if (bind(listenfdv6, (struct sockaddr *)&procmblock->clients[LISTEN_PROCESS_INDEX].addressv6, sizeof(struct sockaddr_in6)) == -1) {
			printf("[-] Failed to bind listen socket for incoming TCP connections: %s\n", strerror(errno));
			exit(EXIT_FAILURE);
		}
		
		helperclient.type = CLIENT_TYPE_TCP_LISTEN_HELPER;
		helperclient.family = CLIENT_FAMILY_HELPER;
		helperclient.pid = getpid();
		helperclient.socketfd = listenfdv6;
		memcpy(&helperclient.address, &procmblock->clients[LISTEN_PROCESS_INDEX].address, sizeof(struct sockaddr_in));
		memcpy(&helperclient.addressv6, &procmblock->clients[LISTEN_PROCESS_INDEX].addressv6, sizeof(struct sockaddr_in6));
		snprintf(helperclient.useragent, sizeof(helperclient.useragent), "TCP listen helper process");
		helperclient.pidfilter = 0;
		helperclient.transmittedbytes = 0;
		
		if ((helperindex = add_client(&helperclient)) == -1) {
			printf("[-] Failed to add helper client info into storage\n");
			close(helperclient.socketfd);
			exit(EXIT_FAILURE);
		}
		
		if (listen(listenfdv6, (int)procmblock->tcpqueue) == -1) {
			printf("[-] Failed to listen socket: %s\n", strerror(errno));
			exit(EXIT_FAILURE);
		}
				
		sockets[0].fd = listenfdv6;
		sockets[0].events = POLLIN;
		
		while (!tlpexitflag) {
			if ((readysockets = poll(sockets, 1, INFTIM)) > 0) {
				if (sockets[0].revents & POLLIN) {
					client.type = CLIENT_TYPE_TCP_UNICAST;
					client.family = CLIENT_FAMILY_V6_TRANSPORT;
					client.pid = 0;
					if ((client.socketfd = accept(listenfdv6, NULL, NULL)) == -1) {
						printf("[-] Connection closed\n");
						close(client.socketfd);
						continue;
					}
					
					if (getpeername(client.socketfd, (struct sockaddr *)&client.addressv6, &addressv6len) == -1) {
						printf("[-] Failed to get client address\n");
					}
					
					memset(recvbuffer, 0, sizeof(recvbuffer));
					if (recv(client.socketfd, recvbuffer, sizeof(recvbuffer), 0) <= 0) {
						printf("[-] Connection closed\n");
						close(client.socketfd);
						continue;
					} else {
						if (strstr((char *)&recvbuffer, HTTPREQUEST) != NULL) {
							if (parse_request((char *)&recvbuffer, (char *)&client.useragent, sizeof(client.useragent), HTTPREQUEST, " HTTP")) {
								client.pidfilter = atoi(client.useragent);
								if (!streamserver_find_pid(client.pidfilter)) {
									client.transmittedbytes = sprintf((char *)&recvbuffer, HTTPANSWER404);
									recvbuffer[client.transmittedbytes] = '\0';
									send(client.socketfd, recvbuffer, client.transmittedbytes, 0);
									printf("[-] Connection closed: unable to find requested resource\n");
									close(client.socketfd);
									continue;
								}
							} else {
								client.pidfilter = 0;
							}
							
							if (!parse_request((char *)&recvbuffer, (char *)&client.useragent, sizeof(client.useragent), "User-Agent: ", "\r\n")) {
								sprintf(client.useragent, "%s", "Unknown client");
							}
							
							if (streamserver_get_free_tcp_client_slots() == 0) {
								client.transmittedbytes = sprintf((char *)&recvbuffer, HTTPANSWER503);
								recvbuffer[client.transmittedbytes] = '\0';
								send(client.socketfd, recvbuffer, client.transmittedbytes, 0);
								printf("[-] Connection closed: server busy\n");
								close(client.socketfd);
								continue;
							} else {
								client.transmittedbytes = sprintf((char *)&recvbuffer, HTTPANSWER200);
								recvbuffer[client.transmittedbytes] = '\0';
								if (send(client.socketfd, recvbuffer, client.transmittedbytes, 0) < 0) {
									printf("[-] Connection closed\n");
									close(client.socketfd);
									continue;
								}
								
								vec[0].iov_base = cmdbuf;
								vec[0].iov_len = 1;
								vec[1].iov_base = &client;
								vec[1].iov_len = sizeof(struct sclient);
								msg.msg_iov = vec;
								msg.msg_iovlen = 2;
								msg.msg_name = NULL;
								msg.msg_namelen = 0;
								
								msg.msg_control = ccmsg;
								msg.msg_controllen = sizeof(ccmsg);
								cmsg = CMSG_FIRSTHDR(&msg);
								cmsg->cmsg_level = SOL_SOCKET;
								cmsg->cmsg_type = SCM_RIGHTS;
								cmsg->cmsg_len = CMSG_LEN(sizeof(client.socketfd));
								*(int*)CMSG_DATA(cmsg) = client.socketfd;
								msg.msg_controllen = cmsg->cmsg_len;
								msg.msg_flags = 0;
								
								sendmsg(fdtransport[1], &msg, 0);
								close(client.socketfd);
							}
						}
					}
				}
			}
		}
		
		close(helperclient.socketfd);
		
		if (remove_client(helperclient.pid) == -1) {
			printf("[-] No client removed - seems like internal error\n");
		}
		
		sigprocmask(SIG_SETMASK, &oldmask, NULL);
		
		exit(EXIT_SUCCESS);
	}
}

int streamserver_stop_tcp_listen_process(void)
{
	if (!streamserver_process_exists(procmblock->clients[LISTEN_PROCESS_INDEX].pid)) {
		printf("[+] TCP LISTEN PROCESS already stopped - need to be started first\n");
		return 0;
	}	
	
	kill(procmblock->clients[LISTEN_PROCESS_INDEX].pid, SIGTERM);

	return 1;
}

int streamserver_init_tcp_unicast(void)
{
	struct sclient client;
	pid_t pid;
	struct msghdr msg;
	struct iovec iov[2];
	char cmdbuf[1];
	char ccmsg[CMSG_SPACE(sizeof(int))];
	struct cmsghdr *cmsg;
	
	iov[0].iov_base = cmdbuf;
	iov[0].iov_len = 1;
	iov[1].iov_base = &client;
	iov[1].iov_len = sizeof(struct sclient);
	msg.msg_iov = iov;
	msg.msg_iovlen = 2;
	msg.msg_name = NULL;
	msg.msg_namelen = 0;
	msg.msg_control = ccmsg;
	msg.msg_controllen = sizeof(ccmsg);
	
	if (recvmsg(fdtransport[0], &msg, 0) == -1) {
		perror("[-] FDTRANSPORT recvmsg error");
	}
	
	switch (cmdbuf[0]) {
		case FDTRANSPORT_NEW_TCP_CLIENT: {
			cmsg = CMSG_FIRSTHDR(&msg);
			if (cmsg->cmsg_type == SCM_RIGHTS) {
				client.socketfd = *(int*)CMSG_DATA(cmsg);
				
				if ((pid = fork()) < 0) {
					//error
					printf("[-] Error in fork\n");
					close(client.socketfd);
				} else if (pid == 0) {
					printf("[+] TCP UNICAST client accepted (PID: %i)\n", client.pidfilter);
					transmission_process(&client);
				}
				
			} else {
				printf("[-] FDTRANSPORT failed to send descriptor\n");
			}
			break;
		} 
		
		default: break;
	}
	
	
}

int streamserver_init_udp_broadcast(struct sockaddr_in *bcaddr)
{
	int bcfd;
	const int broadcast = 1;
	pid_t pid;
	
	int socketfd;
	struct ifconf ifc;
	struct ifreq  *ifr, *currentifr;
	int ifcbuflen;
	char *ifcbuf;
	unsigned int ifcount;
	int i;
	
	struct sclient client;
	
	if (procmblock->clients[UDP_BC_PROCESS_INDEX].socketfd != 0) {
		printf("[-] UDP BROADCAST already running - need to be stopped first\n");
		return 0;
	}
		
	if ((bcfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		printf("[-] Failed to create socket for UDP broadcast: %s\n", strerror(errno));
		close(bcfd);
		return 0;
	}
	
	if (strcmp(procmblock->clients[UDP_BC_PROCESS_INDEX].ifname, "lo") != 0) {
		if ((socketfd = socket(AF_INET, SOCK_DGRAM, 0)) != -1) {
			ifcbuflen = 100 * sizeof(struct ifreq);
			
			while (1) {
				ifcbuf = malloc(ifcbuflen);
				ifc.ifc_len = ifcbuflen;
				ifc.ifc_buf = ifcbuf;
				
				if (ioctl(socketfd, SIOCGIFCONF, &ifc) == -1) {
					if (errno != EINVAL) perror("IOCTL SIOCGIFCONF error");
				} else {
					ifcbuflen = ifc.ifc_len;
					break;
				}		 
				ifcbuflen += 10 * sizeof(struct ifreq);
				free(ifcbuf);
			}
			
			ifr = ifc.ifc_req;
			ifcount = ifc.ifc_len / sizeof(struct ifreq);
			
			for (i=0; i<ifcount; i++) {
				currentifr = &ifr[i];
				
				if (strcmp(procmblock->clients[UDP_BC_PROCESS_INDEX].ifname, currentifr->ifr_name) == 0) {
					if (ioctl(socketfd, SIOCGIFFLAGS, currentifr) != -1) {
						if ((currentifr->ifr_flags & IFF_UP) && (currentifr->ifr_flags & IFF_BROADCAST)) {
							if (ioctl(socketfd, SIOCGIFBRDADDR, currentifr) != -1) {
								memcpy(bcaddr, (struct sockaddr_in *)&currentifr->ifr_broadaddr, sizeof(struct sockaddr_in));
							}
						}
					}
				}
				
			}
			
			free(ifcbuf);
			close(socketfd);
		}
	}
	
	if (setsockopt(bcfd, SOL_SOCKET, SO_BROADCAST, &broadcast, sizeof(broadcast)) == -1) {
		printf("[-] Failed to set UDP socket broadcast option: %s\n", strerror(errno));
		close(bcfd);
		return 0;
	}
	
	client.type = CLIENT_TYPE_UDP_BROADCAST;
	client.family = CLIENT_FAMILY_V4_TRANSPORT;
	client.pid = 0;
	client.socketfd = bcfd;
	memcpy((struct sockaddr_in *)&client.address, bcaddr, sizeof(struct sockaddr_in));
	snprintf(client.useragent, sizeof(client.useragent), "UDP broadcast process");
	client.pidfilter = 0;
	client.transmittedbytes = 0;
	
	if (!streamserver_process_exists(procmblock->clients[UDP_BC_PROCESS_INDEX].pid)) {
		if ((pid = fork()) < 0) {
			//error
			printf("[-] UDP BROADCAST error in fork system call\n");
			return 0;
		} else if (pid == 0) {
			//child
			close(fdtransport[0]);
			transmission_process(&client);
		}
	}
	
	return 1;
}

int streamserver_stop_udp_broadcast(void)
{
	if (!streamserver_process_exists(procmblock->clients[UDP_BC_PROCESS_INDEX].pid)) {
		printf("[+] UDP BROADCAST already stopped - need to be started first\n");
		return 0;
	}	
	
	kill(procmblock->clients[UDP_BC_PROCESS_INDEX].pid, SIGTERM);

	return 1;
}

int streamserver_init_udp_multicast(struct sockaddr_in *udpmgroup)
{
	int mcfd;
	struct in_addr udpmlocal;
	pid_t pid;
	
	int socketfd;
	struct ifconf ifc;
	struct ifreq  *ifr, *currentifr;
	int ifcbuflen;
	char *ifcbuf;
	unsigned int ifcount;
	int i;
	
	unsigned char ttl = (unsigned char)procmblock->udpmcttl;
	unsigned char loop = (unsigned char)procmblock->udpmcloop;
	
	struct sclient client;
	
	if (procmblock->clients[UDP_MC_PROCESS_INDEX].socketfd != 0) {
		printf("[-] UDP MULTICAST already running - need to be stopped first\n");
		return 0;
	}
	
	if ((mcfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		printf("[-] Failed to create socket for UDP multicast: %s\n", strerror(errno));
		close(mcfd);
		return 0;
    }
	
	udpmlocal.s_addr = INADDR_ANY;
	
	if (strcmp(procmblock->clients[UDP_MC_PROCESS_INDEX].ifname, "lo") != 0) {
		if ((socketfd = socket(AF_INET, SOCK_DGRAM, 0)) != -1) {
			ifcbuflen = 100 * sizeof(struct ifreq);
			
			while (1) {
				ifcbuf = malloc(ifcbuflen);
				ifc.ifc_len = ifcbuflen;
				ifc.ifc_buf = ifcbuf;
				
				if (ioctl(socketfd, SIOCGIFCONF, &ifc) == -1) {
					if (errno != EINVAL) perror("IOCTL SIOCGIFCONF error");
				} else {
					ifcbuflen = ifc.ifc_len;
					break;
				}		 
				ifcbuflen += 10 * sizeof(struct ifreq);
				free(ifcbuf);
			}
			
			ifr = ifc.ifc_req;
			ifcount = ifc.ifc_len / sizeof(struct ifreq);
			
			for (i=0; i<ifcount; i++) {
				currentifr = &ifr[i];
				
				if (strcmp(procmblock->clients[UDP_MC_PROCESS_INDEX].ifname, currentifr->ifr_name) == 0) {
					if (ioctl(socketfd, SIOCGIFFLAGS, currentifr) != -1) {
						if ((currentifr->ifr_flags & IFF_UP) && (currentifr->ifr_flags & IFF_MULTICAST)) {
							memcpy((struct sockaddr_in *)&udpmlocal.s_addr, (struct sockaddr_in *)&currentifr->ifr_addr, sizeof(struct sockaddr_in));
						}
					}
				}
				
			}
			
			free(ifcbuf);
			close(socketfd);
		}
	}
	
	// Set the outgoing interface for multicast
	if (setsockopt(mcfd, IPPROTO_IP, IP_MULTICAST_IF, (char *)&udpmlocal, sizeof(struct in_addr)) == -1) {
		printf("[-] Failed to set local interface for UDP multicast: %s\n", strerror(errno));
		close(mcfd);
		return 0;
	}
	
	// Set multicast packet TTL
	if (setsockopt(mcfd, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(unsigned char)) == -1) {
		printf("[-] Failed to set UDP multicast TTL: %s\n", strerror(errno));
	}
	
	// Set multicast traffic loopback
	if (setsockopt(mcfd, IPPROTO_IP, IP_MULTICAST_LOOP, &loop, sizeof(unsigned char)) == -1) {
		printf("[-] Failed to set UDP multicast loopback: %s\n", strerror(errno));
	}
	
	client.type = CLIENT_TYPE_UDP_MULTICAST;
	client.family = CLIENT_FAMILY_V4_TRANSPORT;
	client.pid = 0;
	client.socketfd = mcfd;
	memcpy(&client.address, udpmgroup, sizeof(struct sockaddr_in));
	snprintf(client.useragent, sizeof(client.useragent), "UDP multicast process");
	client.pidfilter = 0;
	client.transmittedbytes = 0;
	
	if (!streamserver_process_exists(procmblock->clients[UDP_MC_PROCESS_INDEX].pid)) {
		if ((pid = fork()) < 0) {
			//error
			printf("[-] UDP MULTICAST error in fork system call\n");
			return 0;
		} else if (pid == 0) {
			//child
			close(fdtransport[0]);
			transmission_process(&client);
		}
	}
	
	return 1;
}

int streamserver_stop_udp_multicast(void)
{
	if (!streamserver_process_exists(procmblock->clients[UDP_MC_PROCESS_INDEX].pid)) {
		printf("[+] UDP MULTICAST already stopped - need to be started first\n");
		return 0;
	}	
	
	kill(procmblock->clients[UDP_MC_PROCESS_INDEX].pid, SIGTERM);

	return 1;
}

int streamserver_init_record(void)
{
	char *homedir;
	time_t utime;
	char tmstr[32];
    struct tm *prtime;
	char filename[PATH_MAX];
	int fd;
	struct msghdr msg;
	char ccmsg[CMSG_SPACE(sizeof(int))];
	struct cmsghdr *cmsg;
	struct iovec vec[1];
	char cmdbuf[1] = {FDTRANSPORT_START_RECORD};
	
	
	if (procmblock->recording) {
		printf("[-] Record already running - need to be stopped first\n");
		return 0;
	}
	
	if (strlen(procmblock->servicename) == 0) 
		sprintf(procmblock->servicename, "%s", "Current service");
	
	homedir = strdup(getenv("HOME"));
	
	utime = time(NULL);
	prtime = localtime(&utime);
	memset(tmstr, 0, sizeof(tmstr));   
    strftime(tmstr, 256, "%F_%T", prtime);
	
	memset(filename, 0, sizeof(filename));
	sprintf((char *)filename, "%s/%s_%s.ts", homedir, procmblock->servicename, tmstr);
	free(homedir);
	
	if ((fd = open((char *)filename, O_CREAT | O_WRONLY, S_IRWXU)) < 0) {
		perror("[-] Open file for recording");
		return 0;
	}
	
	vec[0].iov_base = cmdbuf;
	vec[0].iov_len = 1;
	msg.msg_iov = vec;
	msg.msg_iovlen = 1;
	msg.msg_name = NULL;
	msg.msg_namelen = 0;
	
	msg.msg_control = ccmsg;
	msg.msg_controllen = sizeof(ccmsg);
	cmsg = CMSG_FIRSTHDR(&msg);
	cmsg->cmsg_level = SOL_SOCKET;
	cmsg->cmsg_type = SCM_RIGHTS;
	cmsg->cmsg_len = CMSG_LEN(sizeof(fd));
	*(int*)CMSG_DATA(cmsg) = fd;
	msg.msg_controllen = cmsg->cmsg_len;
	msg.msg_flags = 0;
	
	if (sendmsg(fdtransport[0], &msg, 0) != -1) {
		procmblock->recording = 1;
		close(fd);
		printf("[+] Started record CH: %s, FN: %s\n", procmblock->servicename, filename);
		return 1;
	} else {
		procmblock->recording = 0;
		close(fd);
		perror("[-] Open file for recording");
		return 0;
	}
}

int streamserver_stop_record(void)
{
	struct msghdr msg;
	char ccmsg[CMSG_SPACE(sizeof(int))];
	struct cmsghdr *cmsg;
	struct iovec vec[1];
	char cmdbuf[1] = {FDTRANSPORT_STOP_RECORD};
	
	
	if (!procmblock->recording) {
		printf("[-] Record not running - need to be started first\n");
		return 0;
	}
	
	vec[0].iov_base = cmdbuf;
	vec[0].iov_len = 1;
	msg.msg_iov = vec;
	msg.msg_iovlen = 1;
	msg.msg_name = NULL;
	msg.msg_namelen = 0;
	
	msg.msg_control = NULL;
	msg.msg_controllen = 0;
	msg.msg_flags = 0;
	
	if (sendmsg(fdtransport[0], &msg, 0) != -1) {
		procmblock->recording = 0;
		printf("[+] Record stopped\n");
		return 1;
	} else {
		perror("[-] FDTRANSPORT sendmsg error");
		return 0;
	}
}

int streamserver_init_csa_decoder(int caid, char *key)
{
	int i;
	char hexdigit[3];
	
	if (procmblock->encrypted) {
		printf("Decoder already enabled, changing key\n");
		procmblock->encrypted = 0;
	}
		
	switch (caid) {
		case 0x2600: {
			if (strlen(key) == 16) {
				memset(procmblock->csakey, 0, sizeof(procmblock->csakey));
				for (i=0; i<8; i++) {
					memset(hexdigit, 0, sizeof(hexdigit));
					strncpy(hexdigit, &key[i*2], 2);
					sscanf(hexdigit, "%x", &procmblock->csakey[i]);
					procmblock->csakey[3] = (procmblock->csakey[0] + procmblock->csakey[1] + procmblock->csakey[2]) & 0xff;
					procmblock->csakey[7] = (procmblock->csakey[4] + procmblock->csakey[5] + procmblock->csakey[6]) & 0xff;
				}				
			}
			procmblock->encrypted = 1;
			printf("[+] Encryption: BISS\n");				
			break;
		}
		
		case 0x0000: break;
		
		default: {
			printf("[-] Encryption system not supported\n");
			return 0;
		}
	}
	
	return 1;
}

int streamserver_stop_csa_decoder(void)
{
	if (!procmblock->encrypted) {
		printf("[-] Decoder not enabled\n");
		return 0;
	} else {
		procmblock->encrypted = 0;
		memset(procmblock->csakey, 0, sizeof(procmblock->csakey));
		printf("[-] Decoder disabled\n");
		return 1;
	}
}

int streamserver_open_device(int adapter, int frontend)
{
	char dev_fn[PATH_MAX];
	int fd = -1;
	struct msghdr msg;
	char ccmsg[CMSG_SPACE(sizeof(int))];
	struct cmsghdr *cmsg;
	struct iovec vec[1];
	char cmdbuf[1] = {FDTRANSPORT_CHANGE_DEVICE};
	
	memset(dev_fn, 0, sizeof(dev_fn));
	sprintf(dev_fn, "/dev/dvb/adapter%i/dvr%i", adapter, frontend);
	
	if (access(dev_fn, R_OK) != 0) {
		printf("[-] No access to DVR device %s\n", dev_fn);
		return -1;
	}
	
	if ((fd = open(dev_fn, O_RDONLY | O_NONBLOCK)) <= 0) {
		printf("[-] Failed to open DVR device %s\n", dev_fn);
		return -1;
	}
	
	vec[0].iov_base = cmdbuf;
	vec[0].iov_len = 1;
	msg.msg_iov = vec;
	msg.msg_iovlen = 1;
	msg.msg_name = NULL;
	msg.msg_namelen = 0;
	
	msg.msg_control = ccmsg;
	msg.msg_controllen = sizeof(ccmsg);
	cmsg = CMSG_FIRSTHDR(&msg);
	cmsg->cmsg_level = SOL_SOCKET;
	cmsg->cmsg_type = SCM_RIGHTS;
	cmsg->cmsg_len = CMSG_LEN(sizeof(fd));
	*(int*)CMSG_DATA(cmsg) = fd;
	msg.msg_controllen = cmsg->cmsg_len;
	msg.msg_flags = 0;
	
	if (sendmsg(fdtransport[0], &msg, 0) != -1) {
		close(fd);
		printf("[+] DVR device %s opened\n", dev_fn);
		return 1;
	} else {
		close(fd);
		perror("[-] FDTRANSPORT sendmsg error");
		return 0;
	}
}

void streamserver_read_process_signal_handler(int sig, siginfo_t *si, void *unused)
{
	if ((sig == SIGINT) || (sig == SIGTERM)) {
		rpexitflag = 1;
	}
}

static void init_read_process(void)
{
	struct sigaction sa;
	pid_t pid;
	sigset_t blockmask, oldmask;
	struct sclient readclient;
	unsigned int readindex;
	int dsignal = SIGTERM;
	struct pollfd sockets[2];
	int readysockets;
	struct stat statbuf;
	int deviceready = 0;
	int bufptr = 0;
	int readb = 0;
	int i;
	int recordfd = -1;
	int devicefd = -1;
	int tempfd = -1;
	struct msghdr msg;
	struct iovec iov[1];
	char cmdbuf[1];
	char ccmsg[CMSG_SPACE(sizeof(recordfd))];
	struct cmsghdr *cmsg;
	
	if ((pid = fork()) < 0) {
		//error
		printf("[-] READ PROCESS error in fork system call\n");
		exit(EXIT_FAILURE);
	} else if (pid == 0) {
		//child
		close(fdtransport[0]);
		
		sigemptyset(&blockmask);
		sigaddset(&blockmask, SIGCONT);
		sigprocmask(SIG_BLOCK, &blockmask, &oldmask);
		
		sa.sa_flags = 0;
		sigemptyset(&sa.sa_mask);
		sa.sa_sigaction = streamserver_read_process_signal_handler;
		
		if (sigaction(SIGINT, &sa, NULL) == -1) {
			printf("[-] Error calling SIGINT handler procedure in client process\n");
		}
		
		if (sigaction(SIGTERM, &sa, NULL) == -1) {
			printf("[-] Error calling SIGINT handler procedure in client process\n");
		}
		
		#ifdef linux
			prctl(PR_SET_PDEATHSIG, dsignal, 0, 0, 0);
		#endif
		
		readclient.type = CLIENT_TYPE_READ_HELPER;
		readclient.family = CLIENT_FAMILY_HELPER;
		readclient.pid = getpid();
		readclient.socketfd = 0;
		memcpy(&readclient.address, &procmblock->clients[READ_PROCESS_INDEX].address, sizeof(struct sockaddr_in));
		memcpy(&readclient.addressv6, &procmblock->clients[READ_PROCESS_INDEX].addressv6, sizeof(struct sockaddr_in6));
		snprintf(readclient.useragent, sizeof(readclient.useragent), "Read helper process");
		readclient.pidfilter = 0;
		readclient.transmittedbytes = 0;
		
		if ((readindex = add_client(&readclient)) == -1) {
			printf("[-] Failed to add READ PROCESS info into storage\n");
			exit(EXIT_FAILURE);
		}
		
		sockets[0].fd = fdtransport[1];
		sockets[0].events = POLLIN | POLLERR;
				
		if ((fstat(devicefd, &statbuf) != -1) && (statbuf.st_mode & S_IFCHR)) {
			sockets[1].fd = devicefd;
			sockets[1].events = POLLIN;
			deviceready = 1;
		}
		
		while (!rpexitflag) {
			if (procmblock->sheduler == SHEDULER_RWLOCK) rwlockshed_writelock(rwlockshed);
			
			//read data and send
			bufptr = 0;
			
			while (bufptr != BUFSIZE) {
				if ((readysockets = poll(sockets, deviceready ? 2 : 1, INFTIM)) > 0) {
					if ((deviceready) && (sockets[1].revents & POLLIN)) {
						if ((readb = read(devicefd, &(procmblock->buffer)[bufptr], BUFSIZE-bufptr)) != -1) {
							bufptr = bufptr + readb;
							if (procmblock->buffer[0] != 0x47) {
								for (i=0;i<bufptr;i++) {
									if (procmblock->buffer[i] == 0x47) {
										memcpy(&procmblock->buffer[0], &procmblock->buffer[i], bufptr-i);
										bufptr = bufptr - i;
										break;
									}
								
								}
								printf("[-] Lost sync, %i bytes dropped (read: %i)\n", i, readb);
							}
						}
					}
					
					
					if ((fdtransport[1] > 0) && ((sockets[0].revents & POLLIN) || (sockets[0].revents & POLLERR))) {
						
						if (sockets[0].revents & POLLERR) break;
						
						iov[0].iov_base = cmdbuf;
						iov[0].iov_len = 1;
						msg.msg_iov = iov;
						msg.msg_iovlen = 1;
						msg.msg_name = NULL;
						msg.msg_namelen = 0;
						msg.msg_control = ccmsg;
						msg.msg_controllen = sizeof(ccmsg);
						
						if (recvmsg(fdtransport[1], &msg, 0) <= 0) break;
						
						switch (cmdbuf[0]) {
							case FDTRANSPORT_START_RECORD:
								cmsg = CMSG_FIRSTHDR(&msg);
								if (cmsg->cmsg_type == SCM_RIGHTS) {
									if (recordfd > 0) close(recordfd);
									recordfd = -1;
									tempfd = *(int*)CMSG_DATA(cmsg);
									if ((fstat(tempfd, &statbuf) != -1) && (statbuf.st_mode & S_IFREG)) {
										recordfd = tempfd;
									} else {
										procmblock->recording = 0;
										printf("[-] FDTRANSPORT wrong record descriptor\n");
									}
								} else {
									procmblock->recording = 0;
									printf("[-] FDTRANSPORT failed to send descriptor\n");
								}
								break;
								
							case FDTRANSPORT_STOP_RECORD:
								if (recordfd > 0) close(recordfd);
								recordfd = -1;
								break;
							
							case FDTRANSPORT_CHANGE_DEVICE:
								cmsg = CMSG_FIRSTHDR(&msg);
								if (cmsg->cmsg_type == SCM_RIGHTS) {
									if (deviceready) close(devicefd);
									deviceready = 0;
									devicefd = *(int*)CMSG_DATA(cmsg);
									if ((fstat(devicefd, &statbuf) != -1) && (statbuf.st_mode & S_IFCHR)) {
										sockets[1].fd = devicefd;
										sockets[1].events = POLLIN;
										deviceready = 1;
									} else {
										printf("[-] FDTRANSPORT wrong device descriptor\n");
									}
								} else {
									printf("[-] FDTRANSPORT failed to send descriptor\n");
								}
								break;
							
							default: break;
						}
					}
				} else if ((errno == EINTR) && (rpexitflag)) break;
			}
			
			procmblock->readyclients = 0;
			if (procmblock->sheduler == SHEDULER_SIGNAL) signalshed_wait_value(signalshed, procmblock->numclients);
			
			if (procmblock->encrypted) {
				bufptr = 0;
				while (bufptr < BUFSIZE-1) {
					csa_decrypt(&(procmblock->buffer)[bufptr], procmblock->csakey);
					bufptr = bufptr + 188;
				}
			}
			
			if (procmblock->numclients > 0) {
				for (i=UDP_BC_PROCESS_INDEX; i<MAXCLIENTS; i++) {
					procmblock->clients[i].ready = 0;
				}
				if (procmblock->sheduler == SHEDULER_SIGNAL) signalshed_continue(signalshed);
			}
			
			if ((procmblock->recording) && (recordfd >= 0)) {
				if (write(recordfd, procmblock->buffer, BUFSIZE) == -1) {
					procmblock->recording = 0;
					close(recordfd);
					recordfd = -1;
					printf("[-] Record stopped: %s\n", strerror(errno));
				}
			}
			
			if ((procmblock->sheduler == SHEDULER_SIGNAL) && (procmblock->numclients > 0) && (!rpexitflag)) signalshed_wait(signalshed);
			if (procmblock->sheduler == SHEDULER_RWLOCK) rwlockshed_writeunlock(rwlockshed);
			
		}
		
		
		close(devicefd);
		
		if (remove_client(readclient.pid) == -1) {
			printf("[-] No client removed - seems like internal error\n");
		}
		
		sigprocmask(SIG_SETMASK, &oldmask, NULL);
		
		exit(EXIT_SUCCESS);
	}
}

/*static void clean_lost_clients(void)
{
	int i;
	static union sigval sigvalue;
	
	for (i=MAXCLIENTS; i>=TCP_FIRST_CLIENT; i--) {
		if (process_exists(procmblock->clients[i].pid)) {
			if (procmblock->clients[i].sentbytes == procmblock->clients[i].sentbytessnap) {
				printf("[+] Stopped idle TCP UNICAST process with PID: %u\n", procmblock->clients[i].pid);
				sigvalue.sival_int = 0;
				sigqueue(procmblock->clients[i].pid, SIGTERM, sigvalue);
			}
			procmblock->clients[i].sentbytessnap = procmblock->clients[i].sentbytes;
		}
	}
}*/	

static void main_signal_handler(int sig, siginfo_t *si, void *unused)
{
	int exit_status;
	int client;
	
	if ((sig == SIGINT) || (sig == SIGTERM)) {
		exitflag = 1;
	} else if (sig == SIGHUP) {
		readconfflag = 1;
	} else if (sig == SIGCHLD) {
		cleanupflag = 1;
		/*if ((client = remove_client(si->si_pid)) == -1) {
			printf("[-] No client removed - seems like internal error\n");
		}*/
		//wait(&exit_status);
	}/* else if (sig == SIGALRM) {
		lostcliflag = 1;
	}*/
}

static void start_daemon(void)
{
	int i, fd0, fd1, fd2;
	pid_t pid, sid;
	struct rlimit rlim;
	struct sigaction sa;
		
	umask(0);
	
	if (getrlimit(RLIMIT_NOFILE, &rlim) == -1) {
		printf("[-] rlimit call error\n");
	}
	
	if ((pid = fork()) < 0) {
		printf("[-] Fork call error\n");
	} else if (pid != 0) {
		exit(0);
	}
	
	sid = setsid();
	
	sa.sa_handler = SIG_IGN;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	
	if (sigaction(SIGHUP, &sa, NULL) < 0) {
		printf("[-] Failed to block SIGHUP\n");
	}
	
	if ((pid = fork()) < 0) {
		printf("[-] Fork call error\n");
	} else if (pid != 0) {
		exit(0);
	}
	
	if (rlim.rlim_max == RLIM_INFINITY) rlim.rlim_max = 1024;
	for (i=0; i<rlim.rlim_max; i++) {
		close(i);
	}
	
	fd0 = open("/dev/null", O_RDWR);
	fd1 = dup(0);
	fd2 = dup(0);
}

int main(int argc, char *argv[])
{
	struct sigaction sa;
	int i;
	sigset_t blockmask, oldmask, zeromask;
	//struct itimerval timerdelay;
	key_t mainkey, shedkey;
	int lockfd;
	struct pollfd sockets[2];
	int readysockets;
	char *homedir;
	int ctlsockfd;
	int opt;
	char daemonize = 0, killdaemon = 0;
	char lockfn[PATH_MAX];
	struct flock lock;
	char string[64];
	
	struct utsname hostinfo;
	
	unsigned short semvalues[2];
	union semun semarg;
	
	struct _streamparams sp;
	
			
	while ((opt = getopt(argc, argv, "dkh")) != -1) {
		switch (opt) {
			case 'd':
				daemonize = 1;
				break;
			case 'k':
				killdaemon = 1;
				break;
			case 'h':
				printf("gsfstrsvr [-d] [-k] [-h]\n\n"
						"COMMANDS:\n"
						"  -d		Run as daemon\n"
						"  -k		Kill daemon if running\n"
						"  -h		Print help and exit\n");
				exit(EXIT_SUCCESS);
			default:
				fprintf(stderr, "Usage: %s [-d] [-k] [-h]\n", argv[0]);
				exit(EXIT_FAILURE);
		}
	}
	
	if (daemonize) 
		start_daemon();
	
	homedir = strdup(getenv("HOME"));
	memset(lockfn, 0, sizeof(lockfn));
	
	sprintf((char *)lockfn, STREAMCONFIG_LOCK_PATH, homedir);
	memset(&lock, 0, sizeof(lock));
		
	if ((lockfd = open(lockfn, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)) == -1) {
		printf("[-] Unable to open lock file\n");
		exit(EXIT_FAILURE);
	}
	
	lock.l_type = F_WRLCK;
	lock.l_whence = SEEK_SET;
	lock.l_start = 0;
	lock.l_len = 0;
	if (fcntl(lockfd, F_SETLK, &lock) == -1) {
		if (errno == EACCES || errno == EAGAIN) {
			if (fcntl(lockfd, F_GETLK, &lock) != -1) {
				if (killdaemon) {
					//send SIGINT to stop daemon
					kill(lock.l_pid, SIGTERM);
					printf("[+] Sent INT signal to process %i\n", lock.l_pid);
					close(lockfd);
					exit(EXIT_SUCCESS);
				} else {
					//running process found
					printf("[-] MAIN PROCESS already running with PID %i\n", lock.l_pid);
					close(lockfd);
					exit(EXIT_FAILURE);
				}
			}
		}
	} else if (!killdaemon) {
		//write pid into lock file
		ftruncate(lockfd, 0);
		memset(string, 0, sizeof(string));
		sprintf(string, "%ld", (long)getpid());
		write(lockfd, string, strlen(string)+1);
	} else if (killdaemon) {
		//no running process found
		printf("[-] MAIN PROCESS not running\n");
		close(lockfd);
		exit(EXIT_FAILURE);
	}
		
	//SVIPC initialization	
	if ((mainkey = ftok(RESOURCE_BINARY_STREAMSERVER, MAIN_PROJECT_ID)) == -1) {
		printf("[-] Failed to generate main SYSVIPC key\n");
		exit(EXIT_FAILURE);
	}
	
	if ((shedkey = ftok(RESOURCE_BINARY_STREAMSERVER, SHED_PROJECT_ID)) == -1) {
		printf("[-] Failed to generate sheduler SYSVIPC key\n");
		exit(EXIT_FAILURE);
	}
	
	shmid = shmget(mainkey, sizeof(struct memblock), 0666 | IPC_CREAT);
	
	procmblock = (struct memblock *)shmat(shmid, 0, 0);
	
	memset(procmblock, 0, sizeof(procmblock));
	
	if (uname(&hostinfo) == -1) {
		printf("[-] Failed to get host name via uname call\n");
		exit(EXIT_FAILURE);
	}
	
	memcpy((char *)&procmblock->nodename, (char *)&hostinfo.nodename, _UTSNAME_NODENAME_LENGTH);
	memcpy((char *)&procmblock->osname, (char *)&hostinfo.sysname, _UTSNAME_SYSNAME_LENGTH);
	memcpy((char *)&procmblock->osrelease, (char *)&hostinfo.release, _UTSNAME_RELEASE_LENGTH);
	
	printf("[+] GSF streaming server started at host %s (%s %s)\n", procmblock->nodename, procmblock->osname, procmblock->osrelease);
	
	clisemid = semget(mainkey, 2, 0666 | IPC_CREAT);
	semvalues[0] = 1;
	semvalues[1] = 1;
	
	semarg.array = (unsigned short *)&semvalues;
	//semarg.val = 1;
	if (semctl(clisemid, 0, SETALL, semarg) == -1) {
		perror("[MAIN PROCESS] semctl call:");
	}
	
	if ((ctlsockfd = svrsettings_init()) == -1) {
		printf("[-] Failed to create control socket and set config options\n");
		exit(EXIT_FAILURE);
	}
	
	//FDTRANSPORT socket pair initialization
	socketpair(AF_LOCAL, SOCK_STREAM, 0, fdtransport);
	
	//read default values from config file
	svrsettings_read_config_file(&sp, 1);
	
	//init sheduler
	switch (sp.sheduler) {
		case SHEDULER_SIGNAL:
			signalshed_init(&signalshed, shedkey, 100);
			printf("[+] Selected sheduler: SIGNAL\n");
			break;
		case SHEDULER_RWLOCK:
			rwlockshed_init(&rwlockshed, shedkey, 100, 500);
			printf("[+] Selected sheduler: RWLOCK\n");
			break;
		default:
			signalshed_init(&signalshed, shedkey, 100);
			printf("[+] Selected sheduler: SIGNAL\n");
			break;
	}
	
	//apply settings
	svrsettings_config_apply_settings(&sp, procmblock);
	
	//start read process main loop	
	init_read_process();
	
	//open device and send descriptor
	streamserver_open_device(procmblock->adapter, procmblock->dvr);
	
	//initialize signal handlers
	sa.sa_flags = SA_SIGINFO;
	#ifdef SA_INTERRUPT
		sa.sa_flags |= SA_INTERRUPT;
	#endif
	
	/*#ifdef SA_NOCLDWAIT
		sa.sa_flags |= SA_NOCLDWAIT;
	#endif*/
	
	sigemptyset(&sa.sa_mask);
	sa.sa_sigaction = main_signal_handler;
		
	if (sigaction(SIGINT, &sa, NULL) == -1) {
		printf("[-] Error calling SIGINT signal handler procedure in main process\n");
	}
	
	if (sigaction(SIGTERM, &sa, NULL) == -1) {
		printf("[-] Error calling SIGTERM signal handler procedure in main process\n");
	}
	
	if (sigaction(SIGCHLD, &sa, NULL) == -1) {
		printf("[-] Error calling SIGCHLD signal handler procedure in main process\n");
	}
	
	if (sigaction(SIGHUP, &sa, NULL) == -1) {
		printf("[-] Error calling SIGHUP signal handler procedure in main process\n");
	}
	
	//printf("Main %i\n", getpgrp());
	
	/*if (sigaction(SIGALRM, &sa, NULL) == -1) {
		printf("[-] Error calling SIGALRM signal handler procedure in main process\n");
	}*/
	
	/*timerdelay.it_value.tv_sec = 10;
	timerdelay.it_value.tv_usec = 0;
	timerdelay.it_interval.tv_sec = 10;
	timerdelay.it_interval.tv_usec = 0;
	
	if (setitimer(ITIMER_REAL, &timerdelay, NULL) > 0) {
		printf("[-] Error setting timer - no lost connection control\n");
	}*/
	
	sigemptyset(&blockmask);
	sigaddset(&blockmask, SIGHUP);
	/*sigaddset(&blockmask, SIGALRM);*/
	sigaddset(&blockmask, SIGCHLD);
	
	sockets[0].fd = ctlsockfd;
	sockets[0].events = POLLIN;
	sockets[1].fd = fdtransport[0];
	sockets[1].events = POLLIN;
	
	while (!exitflag) {
		if ((readysockets = poll(sockets, 2, INFTIM)) > 0) {
			if (sockets[0].revents & POLLIN) {
				svrsettings_read_control_socket(sockets[0].fd, procmblock);
			} else if (sockets[1].revents & POLLIN) {
				streamserver_init_tcp_unicast();
			}
		} else if ((readysockets == -1) && (errno == EINTR)) {
			sigprocmask(SIG_BLOCK, &blockmask, &oldmask);
			if (readconfflag) {
				svrsettings_read_config_file(&sp, 0);
				svrsettings_config_apply_settings(&sp, procmblock);
				readconfflag = 0;
			} else if (cleanupflag) {
				streamserver_cleanup_client_slots();
				cleanupflag = 0;
			}
						
			/* else if (lostcliflag) {
				clean_lost_clients();
				lostcliflag = 0;
			}*/
			
			sigprocmask(SIG_SETMASK, &oldmask, NULL);
		}
	} 
	
	printf("[+] Cleaning up reserved resources\n");
	
	//stop all client processes
	sigemptyset(&blockmask);
	sigaddset(&blockmask, SIGTERM);
	sigprocmask(SIG_BLOCK, &blockmask, &oldmask);
	killpg(getpgrp(), SIGTERM);
	wait(NULL);
	sigprocmask(SIG_SETMASK, &oldmask, NULL);
	
	//destroy sheduler
	switch (procmblock->sheduler) {
		case SHEDULER_SIGNAL:
			signalshed_destroy(signalshed);
			break;
		case SHEDULER_RWLOCK:
			rwlockshed_destroy(rwlockshed);
			break;
		default:
			break;
	}
	
	//delete memory block
	shmdt((void *)procmblock);
	shmctl(shmid, IPC_RMID, 0);
	
	//delete semaphore
	semctl(clisemid, 1, IPC_RMID);
	
	//release lock file	
	unlink(lockfn);
	
	printf("[+] Stopped\n");
	
	exit(EXIT_SUCCESS);
}
