/*
 *      prefwindow.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include "settings.h"
#include "mainwindow.h"
#include "sound.h"

struct sprefwindow {
	//program settings window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *apply;
	GtkWidget *adriver;
	GtkWidget *lang;
	GtkWidget *aparam;
	GtkWidget *alimit;
	GtkWidget *snrsc;
	GtkWidget *agcsc;
	GtkWidget *bersc;
	GtkWidget *lowcolor;
	GtkWidget *hicolor;
	GtkWidget *serverpassword;
	GtkWidget *serverport;
	GtkWidget *serversvclist;
	GtkWidget *startintray;
	GtkWidget *exitonclose;
	GSList *languages;
	GSList *soundmodules;
};
struct sprefwindow prefwindow;

gint prefwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
gint prefwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data);


gint prefwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
    prefwindow.visible = FALSE;
	gtk_widget_destroy(prefwindow.win);
    return FALSE;
}

gint prefwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gint ind = 0;
	struct _soundmodule *curmodule;
	GdkColor lowcolor, hicolor;
	gchar *svrpassword;
	gushort svrport;
	gboolean expsvclist;
	
	//sound
	if (prefwindow.soundmodules != NULL) {
		ind = gtk_combo_box_get_active(GTK_COMBO_BOX(prefwindow.adriver));
		curmodule = g_slist_nth_data(prefwindow.soundmodules, ind);
		sound_modules_set_module(curmodule->id);
	}
	ind = gtk_combo_box_get_active(GTK_COMBO_BOX(prefwindow.lang));
	sound_languages_set_language(ind);
	sound_limit_set((guchar)gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefwindow.alimit)));
	sound_parameter_set((guchar)gtk_combo_box_get_active(GTK_COMBO_BOX(prefwindow.aparam)));
	
	//scales
	settings_append_float("scale_snr", gtk_spin_button_get_value(GTK_SPIN_BUTTON(prefwindow.snrsc)));
	settings_append_float("scale_agc", gtk_spin_button_get_value(GTK_SPIN_BUTTON(prefwindow.agcsc)));
	settings_append_float("scale_ber", gtk_spin_button_get_value(GTK_SPIN_BUTTON(prefwindow.bersc)));
	
	//graph
	gtk_color_button_get_color(GTK_COLOR_BUTTON(prefwindow.lowcolor), &lowcolor);
	gtk_color_button_get_color(GTK_COLOR_BUTTON(prefwindow.hicolor), &hicolor);
	mainwindow_graph_set_colors(&lowcolor, &hicolor);
	
	//server
	svrpassword = g_strdup(gtk_entry_get_text(GTK_ENTRY(prefwindow.serverpassword)));
	svrport = (gushort)gtk_spin_button_get_value(GTK_SPIN_BUTTON(prefwindow.serverport));
	expsvclist = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefwindow.serversvclist));
	if (communication_reinit(svrpassword, svrport, expsvclist)) {
		settings_append_string("local_server_password", svrpassword);
		settings_append_int("local_server_port", svrport);
		settings_append_boolean("local_server_export_services", expsvclist);
	}
	g_free(svrpassword);
	
	//other
	settings_append_boolean("program_start_in_tray", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefwindow.startintray)));
	settings_append_boolean("program_exit_on_close", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefwindow.exitonclose)));
	
	settings_flush();
	
	prefwindow_close(NULL, NULL, NULL);
    return FALSE;
}

gboolean prefwindow_create()
{
    GtkWidget *vbox, *hbox;
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *sounds_cp;
	GtkWidget *adriver_l, *lang_l, *aparam_l, *alimit_l;
	GtkWidget *sctable;
	GtkWidget *scales_cp;
	GtkWidget *snrsc_l, *agcsc_l, *bersc_l;
	GtkWidget *graphtable;
	GtkWidget *graph_cp;
	GtkWidget *lowcolor_l, *hicolor_l;
	GtkWidget *servertable;
	GtkWidget *server_cp;
	GtkWidget *passwd_l, *port_l;
	GtkWidget *othertable;
	GtkWidget *other_cp;
	GtkWidget *notebook;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *evbox;
	GtkWidget *chbox;
	GtkWidget *icon;
	gint i = 0;
	gchar *current;
	gint ind = 0;
	gchar *deflang;
	gchar *tmpstr;
	
	
	if (!prefwindow.visible) {
	  prefwindow.visible = TRUE;	  
	  prefwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  //gtk_window_set_resizable(GTK_WINDOW(prefwindow.win),FALSE);
	  gtk_window_set_modal(GTK_WINDOW(prefwindow.win), TRUE);
	  gtk_window_set_resizable(GTK_WINDOW(prefwindow.win), FALSE);
	  prefwindow.title = _("Program settings");
	  gtk_window_set_position(GTK_WINDOW(prefwindow.win), GTK_WIN_POS_CENTER);
	  gtk_window_set_title(GTK_WINDOW(prefwindow.win), prefwindow.title);
	  gtk_widget_set_size_request(prefwindow.win, 400, 300);
	  gtk_window_set_default_size(GTK_WINDOW(prefwindow.win), 400, 300);
	  
	  vbox = gtk_vbox_new(FALSE, 0);
	  
	  fds = pango_font_description_from_string("14");
	  pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
	  prefwindow.caption = gtk_label_new(_("Program settings"));
	  gtk_widget_modify_font(prefwindow.caption, fds);
	  pango_font_description_free(fds);
	  evbox = gtk_event_box_new();
	  gdk_color_parse("white", &color);
	  gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
	  gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox),TRUE);
	  chbox = gtk_hbox_new(FALSE,0);
	  gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
	  icon = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES,GTK_ICON_SIZE_DIALOG);
	  gtk_box_pack_start(GTK_BOX(chbox),icon,FALSE,FALSE,15);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align),prefwindow.caption);
	  gtk_box_pack_start(GTK_BOX(chbox),align,FALSE,FALSE,0);
	  gtk_container_add(GTK_CONTAINER(evbox),chbox);
	  
	  gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
	  
	  gentable = gtk_table_new(3,2,FALSE);
	  gtk_table_set_row_spacings(GTK_TABLE(gentable),5);
	  
	  adriver_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(adriver_l),_("<b>Audio driver:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), adriver_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,0,1);
	  
	  prefwindow.adriver = gtk_combo_box_new_text();
	  
	  struct _soundmodule *curmodule;
	  guint setmodule;
	  
	  prefwindow.soundmodules = sound_modules_get(&setmodule);
	  if (prefwindow.soundmodules != NULL) {
		  for (i = 0; i<g_slist_length(prefwindow.soundmodules); i++) {
			  curmodule = g_slist_nth_data(prefwindow.soundmodules, i);
			  gtk_combo_box_append_text(GTK_COMBO_BOX(prefwindow.adriver), curmodule->name);
		  }
		gtk_combo_box_set_active(GTK_COMBO_BOX(prefwindow.adriver), setmodule);  
	  } else {
		  gtk_combo_box_append_text(GTK_COMBO_BOX(prefwindow.adriver), "Not found");
		  gtk_combo_box_set_active(GTK_COMBO_BOX(prefwindow.adriver), 0);
		  gtk_widget_set_sensitive(GTK_WIDGET(prefwindow.adriver), FALSE);
	  }
	  
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.adriver);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,0,1);
	  
	  lang_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(lang_l),_("<b>Language:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), lang_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	  
	  prefwindow.lang = gtk_combo_box_new_text();
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.lang);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	  
	  struct _soundlanguage *curlang;
	  guint setlang;
	  prefwindow.languages = sound_languages_get(&setlang);
	  if (prefwindow.languages != NULL) {
		  for (i = 0; i<g_slist_length(prefwindow.languages); i++) {
			  curlang = g_slist_nth_data(prefwindow.languages, i);
			  gtk_combo_box_append_text(GTK_COMBO_BOX(prefwindow.lang), curlang->code);
		  }
	  }
	  gtk_combo_box_append_text(GTK_COMBO_BOX(prefwindow.lang), _("As locale"));
	  gtk_combo_box_set_active(GTK_COMBO_BOX(prefwindow.lang), setlang);
	  
	  
	  aparam_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(aparam_l),_("<b>Parameter:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), aparam_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,2,3);
	  
	  prefwindow.aparam = gtk_combo_box_new_text();
	  gtk_combo_box_append_text(GTK_COMBO_BOX(prefwindow.aparam),_("SNR"));
	  gtk_combo_box_append_text(GTK_COMBO_BOX(prefwindow.aparam),_("AGC"));
	  gtk_combo_box_append_text(GTK_COMBO_BOX(prefwindow.aparam),_("BER"));
	  gtk_combo_box_set_active(GTK_COMBO_BOX(prefwindow.aparam), sound_parameter_get());
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.aparam);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,2,3);
	  
	  alimit_l = gtk_label_new(NULL);
	  gtk_label_set_markup(GTK_LABEL(alimit_l),_("<b>Limit:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), alimit_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,3,4);
	  
	  prefwindow.alimit = gtk_spin_button_new_with_range(0.0, 99.0, 1.0);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefwindow.alimit), (gdouble)sound_limit_get());
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.alimit);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,3,4);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(gentable),10);
	  
	  
	  sctable = gtk_table_new(5,2,FALSE);
	  	  
	  snrsc_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(snrsc_l),_("<b>SNR maximum value:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), snrsc_l);
	  gtk_table_attach_defaults(GTK_TABLE(sctable),align,0,1,0,1);
	  
	  prefwindow.snrsc = gtk_spin_button_new_with_range(1.0, 65535.0, 1.0);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefwindow.snrsc),settings_read_float("scale_snr",100.0));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.snrsc);
	  gtk_table_attach_defaults(GTK_TABLE(sctable),align,1,2,0,1);
	  
  	  agcsc_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(agcsc_l),_("<b>AGC maximum value:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), agcsc_l);
	  gtk_table_attach_defaults(GTK_TABLE(sctable),align,0,1,1,2);
	  
	  prefwindow.agcsc = gtk_spin_button_new_with_range(1.0, 65535.0, 1.0);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefwindow.agcsc),settings_read_float("scale_agc",1000.0));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.agcsc);
	  gtk_table_attach_defaults(GTK_TABLE(sctable),align,1,2,1,2);
	  
	  bersc_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(bersc_l),_("<b>BER maximum value:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), bersc_l);
	  gtk_table_attach_defaults(GTK_TABLE(sctable),align,0,1,2,3);
	  
	  prefwindow.bersc = gtk_spin_button_new_with_range(1.0, 4294967296.0, 1.0);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefwindow.bersc),settings_read_float("scale_ber",4294967296.0));
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.bersc);
	  gtk_table_attach_defaults(GTK_TABLE(sctable),align,1,2,2,3);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(sctable),10);
	  
	  
	  graphtable = gtk_table_new(2,2,FALSE);
	  
	  lowcolor_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(lowcolor_l),_("<b>Low value graph color:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), lowcolor_l);
	  gtk_table_attach_defaults(GTK_TABLE(graphtable),align,0,1,0,1);
	  
	  prefwindow.lowcolor = gtk_color_button_new();
	  tmpstr = settings_read_string("graph_low_color", "white");
	  gdk_color_parse(tmpstr, &color);
	  g_free(tmpstr);
	  gtk_color_button_set_color(GTK_COLOR_BUTTON(prefwindow.lowcolor), &color);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.lowcolor);
	  gtk_table_attach_defaults(GTK_TABLE(graphtable),align,1,2,0,1);
	  
  	  hicolor_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(hicolor_l),_("<b>High value graph color:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), hicolor_l);
	  gtk_table_attach_defaults(GTK_TABLE(graphtable),align,0,1,1,2);
	  
	  prefwindow.hicolor = gtk_color_button_new();
	  tmpstr = settings_read_string("graph_high_color", "green3");
	  gdk_color_parse(tmpstr, &color);
	  g_free(tmpstr);
	  gtk_color_button_set_color(GTK_COLOR_BUTTON(prefwindow.hicolor), &color);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.hicolor);
	  gtk_table_attach_defaults(GTK_TABLE(graphtable),align,1,2,1,2);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(graphtable),10);
	  
	  
	  servertable = gtk_table_new(3,2,FALSE);
	  
	  passwd_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(passwd_l),_("<b>Status server password:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), passwd_l);
	  gtk_table_attach_defaults(GTK_TABLE(servertable),align,0,1,0,1);
	  
	  prefwindow.serverpassword = gtk_entry_new();
	  gtk_entry_set_visibility(GTK_ENTRY(prefwindow.serverpassword), FALSE);
	  tmpstr = settings_read_string("local_server_password", "defaultpassword");
	  gtk_entry_set_text(GTK_ENTRY(prefwindow.serverpassword), tmpstr);
	  g_free(tmpstr);
	  align = gtk_alignment_new(0.0, 0.5, 1.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.serverpassword);
	  gtk_table_attach_defaults(GTK_TABLE(servertable),align,1,2,0,1);
	  
	  port_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(port_l),_("<b>Status server port:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), port_l);
	  gtk_table_attach_defaults(GTK_TABLE(servertable),align,0,1,1,2);
	  
	  prefwindow.serverport = gtk_spin_button_new_with_range(1025.0, 65535.0, 1.0);
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefwindow.serverport), (gdouble)settings_read_int("local_server_port", 5555));
	  align = gtk_alignment_new(0.0, 0.5, 1.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), prefwindow.serverport);
	  gtk_table_attach_defaults(GTK_TABLE(servertable),align,1,2,1,2);
	  
	  prefwindow.serversvclist = gtk_check_button_new_with_label(_("Export services list"));
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefwindow.serversvclist), settings_read_boolean("local_server_export_services", TRUE));
	  gtk_table_attach_defaults(GTK_TABLE(servertable),prefwindow.serversvclist,0,2,2,3);
	  //gtk_table_attach(GTK_TABLE(servertable), prefwindow.serversvclist, 0, 2, 1, 2, GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(servertable),10);
	  
	  
	  othertable = gtk_table_new(3,2,FALSE);
	  
	  prefwindow.startintray = gtk_check_button_new_with_label(_("Start hidden in tray"));
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefwindow.startintray), settings_read_boolean("program_start_in_tray", FALSE));
	  gtk_table_attach_defaults(GTK_TABLE(othertable),prefwindow.startintray,0,2,0,1);
	  //gtk_table_attach(GTK_TABLE(othertable), prefwindow.startintray, 0, 2, 0, 1, GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	  
	  prefwindow.exitonclose = gtk_check_button_new_with_label(_("Exit on main window close"));
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefwindow.exitonclose), settings_read_boolean("program_exit_on_close", FALSE));
	  gtk_table_attach_defaults(GTK_TABLE(othertable),prefwindow.exitonclose,0,2,1,2);
	  //gtk_table_attach(GTK_TABLE(othertable), prefwindow.exitonclose, 0, 2, 1, 2, GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(othertable),10);
	  
	  notebook = gtk_notebook_new();
	  sounds_cp = gtk_label_new(_("Sounds"));
	  scales_cp = gtk_label_new(_("Scales"));
	  graph_cp = gtk_label_new(_("Graph"));
	  server_cp = gtk_label_new(_("Status server"));
	  other_cp = gtk_label_new(_("Other"));
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),gentable,sounds_cp);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),sctable,scales_cp);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),graphtable,graph_cp);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),servertable,server_cp);
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),othertable,other_cp);
	  
	  gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
	  
	  prefwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	  prefwindow.apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
	  	  
	  hbox = gtk_hbox_new(FALSE, 0);
	  gtk_box_pack_end(GTK_BOX(hbox), prefwindow.apply, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), prefwindow.close, FALSE, FALSE, 5);
	  
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);

      g_signal_connect(G_OBJECT(prefwindow.win), "destroy",
                       G_CALLBACK(prefwindow_close), G_OBJECT(prefwindow.win));
	  
	  g_signal_connect(G_OBJECT(prefwindow.close), "clicked",
                       G_CALLBACK(prefwindow_close), G_OBJECT(prefwindow.win));
					   
	  g_signal_connect(G_OBJECT(prefwindow.apply), "clicked",
                       G_CALLBACK(prefwindow_apply), G_OBJECT(prefwindow.win));
 
 	  gtk_container_add(GTK_CONTAINER(prefwindow.win),vbox);
	  gtk_widget_show_all(prefwindow.win);
	  
	  gtk_window_set_focus(GTK_WINDOW(prefwindow.win), prefwindow.apply);
	  
  } else {
	  
	  gtk_window_present(GTK_WINDOW(prefwindow.win));
	  
  }
	  
      return TRUE; 
}
