/*
 *      streamwindow.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include "streamconfig.h"

#define UDP_GROUP_REGEX "^(22[4-9]|23[0-9])(\\.(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])){3}$"

struct sstreamwindow {
	//streaming server settings window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *apply;
	GtkWidget *udpbcint;
	GtkWidget *udpbcp;
	GtkWidget *udpmcint;
	GtkWidget *udpmcgroup;
	GtkWidget *udpmcp;
	GtkWidget *udpmcloop;
	GtkWidget *udpmcttl;
	GtkWidget *tcpucq;
	GtkWidget *tcpucc;
	GtkWidget *tcpucp;
	GRegex *mcipregex;
	gboolean regex_lock;
	struct _streamconfig *sc;
};
struct sstreamwindow streamwindow;

enum {
	NETINTERFACE_NAME = 0,
	NETINTERFACE_NUMBER,
	NETINTERFACE_COLUMNS
};

static gint streamwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
static gint streamwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data);


static gint streamwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	if (!streamwindow.regex_lock) {
		g_regex_unref(streamwindow.mcipregex);
		streamwindow.regex_lock = TRUE;
	}
    streamwindow.visible = FALSE;
	gtk_widget_destroy(streamwindow.win);
    return FALSE;
}

static gint streamwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	struct _streamconnparams sendparams;
	gchar *groupip;
	gchar *ifname;
	GtkTreeModel *model;
	GtkTreeIter iter;
	
	memset(&sendparams, 0, sizeof(struct _streamconnparams));
		
	sprintf(sendparams.udpbcint, "lo");
	model = gtk_combo_box_get_model(GTK_COMBO_BOX(streamwindow.udpbcint));
	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(streamwindow.udpbcint), &iter)) {
		gtk_tree_model_get(model, &iter, NETINTERFACE_NAME, &ifname, -1);
		sprintf(sendparams.udpbcint, ifname);
		g_free(ifname);
	}
	
	sprintf(sendparams.udpmcint, "lo");
	model = gtk_combo_box_get_model(GTK_COMBO_BOX(streamwindow.udpmcint));
	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(streamwindow.udpmcint), &iter)) {
		gtk_tree_model_get(model, &iter, NETINTERFACE_NAME, &ifname, -1);
		sprintf(sendparams.udpmcint, ifname);
		g_free(ifname);
	}
	
	groupip = g_strdup(gtk_entry_get_text(GTK_ENTRY(streamwindow.udpmcgroup)));
	strncpy(sendparams.udpmcgroup, groupip, strlen(groupip));
	g_free(groupip);
	
	sendparams.udpbcport = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streamwindow.udpbcp));
	sendparams.udpmcport = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streamwindow.udpmcp));
	sendparams.udpmcloop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streamwindow.udpmcloop));
	sendparams.udpmcttl = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streamwindow.udpmcttl));
	sendparams.tcpucport = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streamwindow.tcpucp));
	sendparams.tcpucqueue = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streamwindow.tcpucq));
	sendparams.tcpucclients = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streamwindow.tcpucc));
	
	streamconfig_set_connection_details(streamwindow.sc, &sendparams);
	
	streamwindow_close(NULL, NULL, NULL);
    return FALSE;
}

static void streamwindow_validate_regex(GtkEditable *editable, gpointer data)
{
	gchar *txt;
	GRegex *regex;
	
	txt = g_strdup(gtk_entry_get_text(GTK_ENTRY(editable)));
	regex = g_regex_ref(streamwindow.mcipregex);
	
	if (!g_regex_match_all(regex, txt, 0, NULL)) {
		gtk_widget_set_sensitive(streamwindow.apply, FALSE);
	} else {
		gtk_widget_set_sensitive(streamwindow.apply, TRUE);
	}
	
	g_regex_unref(regex);
	g_free(txt);
}

void streamwindow_interfaces_foreach(gpointer data, GtkListStore **store)
{
	GtkTreeIter iter;
	struct _netinterface *interface;
	
	interface = (struct _netinterface *)data;
	
	gtk_list_store_append(GTK_LIST_STORE(*store), &iter);
	gtk_list_store_set(GTK_LIST_STORE(*store), &iter, NETINTERFACE_NAME, interface->ifname, NETINTERFACE_NUMBER, interface->ifindex, -1);
}

gboolean streamwindow_interfaces_select_bc_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	gchar *ifname;
	gchar *srcifname;
	
	srcifname = (gchar *)data;
	
	gtk_tree_model_get(model, iter, NETINTERFACE_NAME, &ifname, -1);
	
	if (g_str_equal(srcifname, ifname)) {
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(streamwindow.udpbcint), iter);
		g_free(ifname);
		return TRUE;
	}
	g_free(ifname);
		
	return FALSE;
}

gboolean streamwindow_interfaces_select_mc_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	gchar *ifname;
	gchar *srcifname;
	
	srcifname = (gchar *)data;
	
	gtk_tree_model_get(model, iter, NETINTERFACE_NAME, &ifname, -1);
	
	if (g_str_equal(srcifname, ifname)) {
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(streamwindow.udpmcint), iter);
		return TRUE;
	}
	
	return FALSE;
}

gboolean streamwindow_create(struct _streamconfig *sc)
{
    GtkWidget *vbox, *hbox;
	GtkWidget *align;
	GtkWidget *udpbctable, *udpmctable, *tcpuctable;
	GtkWidget *udpbcint_l, *udpmcint_l, *tcpucip_l;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *evbox;
	GtkWidget *chbox;
	GtkWidget *icon;
	
	GtkCellRenderer *renderer;
	GtkListStore *intstore;
	GtkTreeIter iter;
	GSList *interfaces;
	
	GtkTreeModel *intmodel;
	
	if (!streamwindow.visible) {
	  streamwindow.visible = TRUE;	  
	  streamwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  streamwindow.sc = sc;
	  gtk_window_set_modal(GTK_WINDOW(streamwindow.win), TRUE);
	  gtk_window_set_resizable(GTK_WINDOW(streamwindow.win), FALSE);
	  streamwindow.title = _("Streaming server settings");
	  gtk_window_set_position(GTK_WINDOW(streamwindow.win), GTK_WIN_POS_CENTER);
	  gtk_window_set_title(GTK_WINDOW(streamwindow.win), streamwindow.title);
	  gtk_widget_set_size_request(streamwindow.win, 400, 300);
	  gtk_window_set_default_size(GTK_WINDOW(streamwindow.win), 400, 300);
	  
	  streamwindow.mcipregex = g_regex_new(UDP_GROUP_REGEX, G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
	  streamwindow.regex_lock = FALSE;
	  
	  vbox = gtk_vbox_new(FALSE, 0);
	  
	  fds = pango_font_description_from_string("14");
	  pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
	  streamwindow.caption = gtk_label_new(_("Streaming server settings"));
	  gtk_widget_modify_font(streamwindow.caption, fds);
	  pango_font_description_free(fds);
	  evbox = gtk_event_box_new();
	  gdk_color_parse("white", &color);
	  gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
	  gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox),TRUE);
	  chbox = gtk_hbox_new(FALSE,0);
	  gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
	  icon = gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_DIALOG);
	  gtk_box_pack_start(GTK_BOX(chbox),icon,FALSE,FALSE,15);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align),streamwindow.caption);
	  gtk_box_pack_start(GTK_BOX(chbox),align,FALSE,FALSE,0);
	  gtk_container_add(GTK_CONTAINER(evbox),chbox);
	  
	  gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
	  
	  GtkWidget *notebook = gtk_notebook_new();
	  
	  udpbctable = gtk_table_new(2,2,FALSE);
	  gtk_table_set_row_spacings(GTK_TABLE(udpbctable),5);
	  
	  udpbcint_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(udpbcint_l),_("<b>Target interface:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), udpbcint_l);
	  gtk_table_attach_defaults(GTK_TABLE(udpbctable),align,0,1,0,1);
	  
	  streamwindow.udpbcint = gtk_combo_box_new();
	  
	  renderer = gtk_cell_renderer_text_new();
	  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(streamwindow.udpbcint), renderer, TRUE);
	  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(streamwindow.udpbcint), renderer, "markup", NETINTERFACE_NAME, NULL);
	  
	  intstore = gtk_list_store_new(NETINTERFACE_COLUMNS, G_TYPE_STRING, G_TYPE_ULONG);
	  
	  if ((interfaces = streamconfig_interfaces_get_with_broadcast()) != NULL) {
		  g_slist_foreach(interfaces, (GFunc)streamwindow_interfaces_foreach, &intstore);
		  g_slist_free(interfaces);
	  } else {
		  gtk_list_store_append(GTK_LIST_STORE(intstore), &iter);
		  gtk_list_store_set(GTK_LIST_STORE(intstore), &iter, NETINTERFACE_NAME, "lo", NETINTERFACE_NUMBER, 1, -1);
	  }
	  	  
	  gtk_combo_box_set_model(GTK_COMBO_BOX(streamwindow.udpbcint), GTK_TREE_MODEL(intstore));
	  g_object_unref(intstore);
	  
	  gtk_combo_box_set_active(GTK_COMBO_BOX(streamwindow.udpbcint), 0);
	  
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.udpbcint);
	  gtk_table_attach_defaults(GTK_TABLE(udpbctable),align,1,2,0,1);
	  
	  GtkWidget *udpbcp_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(udpbcp_l),_("<b>Port number:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), udpbcp_l);
	  gtk_table_attach_defaults(GTK_TABLE(udpbctable),align,0,1,1,2);
	  
	  streamwindow.udpbcp = gtk_spin_button_new_with_range(1024, 65535, 1);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.udpbcp);
	  gtk_table_attach_defaults(GTK_TABLE(udpbctable),align,1,2,1,2);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(udpbctable), 10);
	  
	  GtkWidget *udpbc_cp = gtk_label_new(_("UDP Broadcast"));
	  
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), udpbctable, udpbc_cp);
	  
	  
	  udpmctable = gtk_table_new(5, 3, FALSE);
	  gtk_table_set_row_spacings(GTK_TABLE(udpmctable), 5);
	  
	  udpmcint_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(udpmcint_l),_("<b>Target interface:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), udpmcint_l);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,0,1,0,1);
	  
	  streamwindow.udpmcint = gtk_combo_box_new();
	  
	  renderer = gtk_cell_renderer_text_new();
	  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(streamwindow.udpmcint), renderer, TRUE);
	  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(streamwindow.udpmcint), renderer, "markup", NETINTERFACE_NAME, NULL);
	  
	  intstore = gtk_list_store_new(NETINTERFACE_COLUMNS, G_TYPE_STRING, G_TYPE_ULONG);
	  
	  if ((interfaces = streamconfig_interfaces_get_with_multicast()) != NULL) {
		  g_slist_foreach(interfaces, (GFunc)streamwindow_interfaces_foreach, &intstore);
		  g_slist_free(interfaces);
	  } else {
		  gtk_list_store_append(GTK_LIST_STORE(intstore), &iter);
		  gtk_list_store_set(GTK_LIST_STORE(intstore), &iter, NETINTERFACE_NAME, "lo", NETINTERFACE_NUMBER, 1, -1);
	  }
	  	  
	  gtk_combo_box_set_model(GTK_COMBO_BOX(streamwindow.udpmcint), GTK_TREE_MODEL(intstore));
	  g_object_unref(intstore);
	  
	  gtk_combo_box_set_active(GTK_COMBO_BOX(streamwindow.udpmcint), 0);
	  
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.udpmcint);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,1,3,0,1);
	  
	  udpmcint_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(udpmcint_l),_("<b>Group address:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), udpmcint_l);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,0,1,1,2);
	  
	  streamwindow.udpmcgroup = gtk_entry_new_with_max_length(INET6_ADDRSTRLEN);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.udpmcgroup);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,1,3,1,2);
	  
	  g_signal_connect(G_OBJECT(streamwindow.udpmcgroup), "changed", G_CALLBACK(streamwindow_validate_regex), NULL);
	  	  
	  GtkWidget *udpmcp_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(udpmcp_l),_("<b>Port number:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), udpmcp_l);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,0,1,2,3);
	  	  
	  streamwindow.udpmcp = gtk_spin_button_new_with_range(1024, 65535, 1);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.udpmcp);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,1,3,2,3);
	  
	  GtkWidget *udpmcloop_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(udpmcloop_l),_("<b>Loopback/TTL:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), udpmcloop_l);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,0,1,3,4);
	  
	  streamwindow.udpmcloop = gtk_spin_button_new_with_range(0, 1, 1);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.udpmcloop);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,1,2,3,4);
	  
	  /*GtkWidget *udpmcttl_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(udpmcttl_l),_("<b>Packet TTL:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), udpmcttl_l);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,0,1,4,5);*/
	  
	  streamwindow.udpmcttl = gtk_spin_button_new_with_range(1, 255, 1);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.udpmcttl);
	  gtk_table_attach_defaults(GTK_TABLE(udpmctable),align,2,3,3,4);
	  
	  
	  gtk_container_set_border_width(GTK_CONTAINER(udpmctable), 10);
	  
	  GtkWidget *udpmc_cp = gtk_label_new(_("UDP Multicast"));
	  
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), udpmctable, udpmc_cp);
	  
	  
	  tcpuctable = gtk_table_new(3, 2, FALSE);
	  gtk_table_set_row_spacings(GTK_TABLE(tcpuctable), 5);
	  
	  tcpucip_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(tcpucip_l),_("<b>Clients number:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tcpucip_l);
	  gtk_table_attach_defaults(GTK_TABLE(tcpuctable),align,0,1,0,1);
	  
	  streamwindow.tcpucc = gtk_spin_button_new_with_range(1, 90, 1);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.tcpucc);
	  gtk_table_attach(GTK_TABLE(tcpuctable),align,1,2,0,1,GTK_FILL,GTK_FILL,2,0);
	  
	  GtkWidget *tcpucque_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(tcpucque_l),_("<b>Queue length:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tcpucque_l);
	  gtk_table_attach_defaults(GTK_TABLE(tcpuctable),align,0,1,1,2);
	  
	  streamwindow.tcpucq = gtk_spin_button_new_with_range(1, 10, 1);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.tcpucq);
	  gtk_table_attach(GTK_TABLE(tcpuctable),align,1,2,1,2,GTK_FILL,GTK_FILL,2,0);
	  
	  GtkWidget *tcpucport_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(tcpucport_l),_("<b>Port number:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), tcpucport_l);
	  gtk_table_attach_defaults(GTK_TABLE(tcpuctable),align,0,1,2,3);
	  
	  streamwindow.tcpucp = gtk_spin_button_new_with_range(1024, 65535, 1);
	  align = gtk_alignment_new(1.0, 0.5, 1.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), streamwindow.tcpucp);
	  gtk_table_attach_defaults(GTK_TABLE(tcpuctable),align,1,2,2,3);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(tcpuctable), 10);
	  
	  GtkWidget *tcpuc_cp = gtk_label_new(_("TCP Unicast"));
	  
	  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tcpuctable, tcpuc_cp);
	    
	  
	  gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
	  
	  streamwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	  streamwindow.apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
	  	  
	  hbox = gtk_hbox_new(FALSE, 0);
	  gtk_box_pack_end(GTK_BOX(hbox), streamwindow.apply, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), streamwindow.close, FALSE, FALSE, 5);
	  
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
	  
	  g_signal_connect(G_OBJECT(streamwindow.win), "destroy", G_CALLBACK(streamwindow_close), NULL);
	  g_signal_connect(G_OBJECT(streamwindow.close), "clicked", G_CALLBACK(streamwindow_close), NULL);
	  g_signal_connect(G_OBJECT(streamwindow.apply), "clicked", G_CALLBACK(streamwindow_apply), NULL);
	   
 	  gtk_container_add(GTK_CONTAINER(streamwindow.win),vbox);
	  
	  struct _streamconnparams recvparams;
	  
	  if (streamconfig_get_connection_details(sc, &recvparams)) {
		  if (recvparams.udpbcint[0] != '\0') {
			  intmodel = gtk_combo_box_get_model(GTK_COMBO_BOX(streamwindow.udpbcint));
			  gtk_tree_model_foreach(intmodel, streamwindow_interfaces_select_bc_foreach, &recvparams.udpbcint);
		  }
		  
		  if (recvparams.udpmcint[0] != '\0') {
			  intmodel = gtk_combo_box_get_model(GTK_COMBO_BOX(streamwindow.udpmcint));
			  gtk_tree_model_foreach(intmodel, streamwindow_interfaces_select_mc_foreach, &recvparams.udpmcint);
		  }
		  
		  if (recvparams.udpmcgroup[0] != '\0') gtk_entry_set_text(GTK_ENTRY(streamwindow.udpmcgroup), recvparams.udpmcgroup);
		  
		  gtk_spin_button_set_value(GTK_SPIN_BUTTON(streamwindow.udpbcp), (gdouble)recvparams.udpbcport);
		  gtk_spin_button_set_value(GTK_SPIN_BUTTON(streamwindow.udpmcp), (gdouble)recvparams.udpmcport);
		  gtk_spin_button_set_value(GTK_SPIN_BUTTON(streamwindow.udpmcloop), (gdouble)recvparams.udpmcloop);
		  gtk_spin_button_set_value(GTK_SPIN_BUTTON(streamwindow.udpmcttl), (gdouble)recvparams.udpmcttl);
		  gtk_spin_button_set_value(GTK_SPIN_BUTTON(streamwindow.tcpucp), (gdouble)recvparams.tcpucport);
		  gtk_spin_button_set_value(GTK_SPIN_BUTTON(streamwindow.tcpucq), (gdouble)recvparams.tcpucqueue);
		  gtk_spin_button_set_value(GTK_SPIN_BUTTON(streamwindow.tcpucc), (gdouble)recvparams.tcpucclients);
	  }
	  	  
	  gtk_widget_show_all(streamwindow.win);
	  
	  gtk_window_set_focus(GTK_WINDOW(streamwindow.win), streamwindow.apply);
	  
  } else {
	  
	  gtk_window_present(GTK_WINDOW(streamwindow.win));
	  
  }
	  
      return TRUE; 
}
